##################################TCL FILE####################################
# Description :
# Postprocessor for machine: FCQV with HEIDENHAIN TNC-360
# Serial number:
# CNC Control: HEIDENHAIN TNC 360
# Software version:
#
# Revisions
#
#   Date             Who			Reason
#  01-Nov-2002          First Release +   
#  20.04.2003          1.  MOM_before_output
#                                       2.    . 
#                                            (  MOM_tool_change)
#                                       3.  MOM_circular_move (     
#                                           (  start_of_path, ..    )+      .
#                                       4.      catch { }
#                                       5.    -> :
#                                       6.       : xxxxxx00.h, xxxxxx01.h, xxxxxx02.h,.....
#                                       7. o MOM_divide:   divide_flag={ON OFF} -      (  =OFF)
#                                       8. o MOM_opskip (OPSKIP_SET)  MOM_opskip_{on off}
#                                       9. o MOM_linear_move ->       ,   M90 & M97
#                                      10. o MOM_start_of_program ->  PB_start_of_program
#                                      11. o MOM_start_of_group ->     PB_start_of_program
#                                      12.  MOM_end_of_program ->    ,    x 4000
#  22.04.2003         1. o MOM_tolerance_cycle -   
#                                      2.  MOM_rotation_cycle ->  
#                                      3.  MOM_datum_shift_cycle ->   
#                                      4.  MOM_scaling_cycle ->   
#                                      5.  MOM_block_create
#  23.04.2003         1.   (  TNC 360   :
#                                            1(PECKING) 2(TAPPING) 17(RIGID TAPPING GS) , .  )
# $HISTORY$
#
###############################################################################
#    %UGII_CAM_POST_DIR%    
#      "ugpost_base.tcl"
set cam_post_dir [MOM_ask_env_var UGII_CAM_POST_DIR]
#    %UGII_CAM_DEBUG_DIR%    
#    
set cam_debug_dir [MOM_ask_env_var UGII_CAM_DEBUG_DIR]
#    ugpost_base.tcl      
source ${cam_post_dir}/ugpost_base.tcl
#       ,     
# source [MOM_ask_env_var UGII_CAM_DEBUG_DIR]mom_debug.tcl ; source [MOM_ask_env_var UGII_CAM_DEBUG_DIR]mom_review.tcl ; MOM_set_debug_mode ON

#   G   
set mom_sys_program_stop_code                 			0
set mom_sys_optional_stop_code                			1
set mom_sys_end_of_program_code               			2
set mom_sys_spindle_direction_code(CLW)       			3
set mom_sys_spindle_direction_code(CCLW)      			4
set mom_sys_spindle_direction_code(OFF)       			5
set mom_sys_coolant_code(MIST)                			8
set mom_sys_coolant_code(ON)                  			8
set mom_sys_coolant_code(TAP)                 			8
set mom_sys_coolant_code(FLOOD)               			8
set mom_sys_coolant_code(OFF)                 			9
set mom_sys_rewind_code                       			30
#     
set mom_kin_arc_output_mode                   			FULL_CIRCLE
set mom_kin_arc_valid_planes                  			XYZ
set mom_kin_clamp_time                        			2.0
set mom_kin_linearization_flag                			1
set mom_kin_linearization_tol                 			0.02
set mom_kin_machine_resolution                			0.001
set mom_kin_machine_type                      			3_axis_mill
set mom_kin_max_arc_radius                    			99999.999
set mom_kin_min_arc_radius                    			0.001
set mom_kin_output_unit                       			MM
set mom_kin_post_data_unit                    			MM
set mom_kin_rapid_feed_rate                   			10000.0
set mom_kin_tool_change_time                  			12.0
set mom_kin_x_axis_limit                      			2000.0 ; # X
set mom_kin_y_axis_limit                      			800.0  ; # Y
set mom_kin_z_axis_limit                      			600.0  ; # Z
set mom_kin_nurbs_output_type					HEIDENHAIN_POLY
#      
set mom_sys_opskip_block_leader                                 "/"
set mom_sys_control_in                                          ""
set mom_sys_control_out                                         "; "
set mom_sys_list_output                       			OFF
set mom_sys_header_output                     			ON
set mom_sys_list_file_rows                    			250
set mom_sys_list_file_columns                 			80
set mom_sys_warning_output                    			OFF
set mom_sys_group_output                      			OFF
set mom_sys_list_file_suffix                  			lpt
set mom_list_file_suffix              				lpt
set mom_sys_output_file_suffix                			H
set mom_output_file_suffix                			H
set mom_spindle_speed						1000.0
#      
set header_check_status						0
set mom_smooth_path						"OFF"
set mom_tolerance_on_off					"OFF"
set mom_small_steps						"OFF"
set status_block_create						0
set status_tool_change					      	0
set spindle_off_flag 						0
set mom_point1_pos(0) 						-100.0
set mom_point1_pos(1) 						-100.0
set mom_point1_pos(2) 						-50.0
set mom_point2_pos(0) 						100.0
set mom_point2_pos(1) 						100.0
set mom_point2_pos(2) 						0.0
set mom_offset_block						10.0
set mom_cycle_cam						"0"
set mom_cycle_orient						"0"
set mom_cycle_step3						0.2
set mom_cycle_delay						"0.1"
set mom_prev_pos(2)						999999.999
set mom_from_pos(0)						0.0
set mom_from_pos(1)						0.0
set mom_from_pos(2)						0.0
set length 							0
set accom_length						0
set max_length							3900 ; #   
set i								1
set mom_divide_by						"LENGTH"
set accumulated_time						0
set mom_retract_to						"OFFSET"
set mom_offset_dist						100.0
set d_time							0
set cutcom_on 							0
set divide_flag                                                 OFF

#=============================================================
proc MOM_start_of_program { } {
#      
#=============================================================
  	global mom_logname mom_date
  	global mom_coolant_status mom_cutcom_status
  	global mom_clamp_status mom_cycle_status
  	global mom_spindle_status mom_cutcom_plane
  	global mom_cutcom_adjust_register mom_tool_adjust_register
  	global mom_tool_length_adjust_register mom_length_comp_register
  	global mom_flush_register mom_wire_cutcom_adjust_register
  	global mom_output_file_basename mom_output_file_suffix

  	set mom_coolant_status UNDEFINED
	set mom_cutcom_status  UNDEFINED
  	set mom_clamp_status   UNDEFINED
  	set mom_cycle_status   UNDEFINED
  	set mom_spindle_status UNDEFINED
  	set mom_cutcom_plane   UNDEFINED

  	catch {unset mom_cutcom_adjust_register}
  	catch {unset mom_tool_adjust_register}
  	catch {unset mom_tool_length_adjust_register}
  	catch {unset mom_length_comp_register}
  	catch {unset mom_flush_register}
  	catch {unset mom_wire_cutcom_adjust_register}

 global pb_start_of_program_flag ;  set pb_start_of_program_flag 0

  	OPEN_files ; #   ( )  
 	LIST_FILE_HEADER ; #    ""

}


#############################################	 
set Num ""
#=============================================================
proc PB_start_of_program { } {
#=============================================================
  global mom_sys_rewind_stop_code
  global mom_output_file_basename  ptp_file_name
  global mom_warning_info
  global mom_sys_list_output

  set mom_warning_info "\n  TNC - 360 ( Heidenhain)\n"
  append mom_warning_info "\t  TNC - 360:\n\t  :   ,  , , ;\n"
  append mom_warning_info "\t   :   -,  (  ),\n"
  append mom_warning_info "\t                           ,       \n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;

 ;#      ( )
  global mom_update_post_cmds_from_tool
  if {[info exists mom_update_post_cmds_from_tool]} {
    set mom_warning_info "\n  mom_update_post_cmds_from_tool=$mom_update_post_cmds_from_tool\n"
    append mom_warning_info "\t=>\n"
    switch $mom_update_post_cmds_from_tool {
    0 { append mom_warning_info "\t1)           'Tool Change'\n"
        append mom_warning_info "\t2)  ,    (),   -     'Spindle On/Off'\n"
        append mom_warning_info "\n\t ! ,        ,\n"
        append mom_warning_info "\t     '' -   'Manufacturing Preferences'-> 'Operation'\n"
        append mom_warning_info "\t    'Update Post From Tool'.\n"
      }
     default {
        append mom_warning_info "\t1)     ,     (  ) -  ,    \n"
        append mom_warning_info "\t2)   ,   -     'Spindle On/Off'\n"
        append mom_warning_info "\n\t ! ,          ,\n"
        append mom_warning_info "\t     '' -   'Manufacturing Preferences'-> 'Operation'\n"
        append mom_warning_info "\t    'Update Post From Tool'.\n"
       }
     }
     MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
   }


 #      , 
 #      

;# set is_name_ok [regexp -nocase -- {[a-z]} $mom_output_file_basename]
;# if {$is_name_ok == "1"} {
;# 	MOM_abort "\n\n\t: (  Heidenhain 'TNC-360',  FSQV NC63)\n\t    -     (max 8)   *.H  *.i\n\n"
;# }

 ; #   
     set var0 "" 
     set fl1 [regexp -- {([0-9])+} $mom_output_file_basename var0]
     if {$fl1=="0"} {
 	MOM_abort "\n\n\t: (  Heidenhain 'TNC-360',  FCQV-63NC)\n\t    -     (max 8)\n\n"
     }
     set var1 ""
     regsub -nocase -- {(0)*} $var0 "" var1 ;
     if {$var1==""} {
 	MOM_abort "\n\n\t: (  Heidenhain 'TNC-360',  FCQV-63NC)\n\t    -     (max 8)\n\n"
     }
     set Num [format "%s" $var1]


 global count ; set count 0 ;
 global fileStr ; set fileStr $Num ;# $mom_output_file_basename
 set str "" ;
 global divide_flag ;
 if {$divide_flag=="ON"} {
     set mom_sys_list_output OFF
     set str [format "%02d" $count] ;
     set mom_output_file_basename "$fileStr$str"
 }

 ;#  	OPEN_files ; #   ( )  
 ;# 	LIST_FILE_HEADER ; #    ""

;#  MOM_close_output_file $ptp_file_name
 if {[file exists $ptp_file_name]} {  MOM_remove_file $ptp_file_name ; }
 MOM_open_output_file $ptp_file_name
 MOM_reset_sequence 1 1 1
 MOM_force Once X Y Z F M_spindle

	global header_check_status
 	if { $header_check_status == 0} {
		MOM_output_literal "BEGIN PGM $fileStr$str MM"
		global prog_name
		set prog_name $fileStr ; # $mom_output_file_basename
		set header_check_status 1
	}

 global block_min block_max
 global status_block_create
  if {[info exists status_block_create]} {
   if {$status_block_create==1} {
	MOM_output_literal "$block_min"
	MOM_output_literal "$block_max"
	set status_block_create 1
    }
  }

}
#############################################

#=============================================================
proc MOM_start_of_group {} {
#      UG   ,
#    
#=============================================================
	global mom_output_file_basename mom_group_name
	global header_check_status mom_operation_name

  global prog_name
  set prog_name ""
  if {[info exists mom_operation_name]} { set prog_name $mom_operation_name }
  if {[info exists mom_group_name]} { set prog_name $mom_group_name }
  
	set prog_name $mom_output_file_basename
}

#=============================================================
proc MOM_block_create { } {
#    2-  3-   ,  
#      
#=============================================================
	global mom_point1_pos
	global mom_point2_pos
	global header_check_status
	global status_block_create
	global mom_offset_block

	MOM_force once X Y Z
	set mom_point1_pos(0) [expr $mom_point1_pos(0)-$mom_offset_block]
	set mom_point1_pos(1) [expr $mom_point1_pos(1)-$mom_offset_block]
	set mom_point1_pos(2) [expr $mom_point1_pos(2)-$mom_offset_block]
	set mom_point2_pos(0) [expr $mom_point2_pos(0)+$mom_offset_block]
	set mom_point2_pos(1) [expr $mom_point2_pos(1)+$mom_offset_block]

        global block_min  block_max
	set block_min [ MOM_do_template start_block CREATE ]
	set block_max [ MOM_do_template end_block CREATE ]

        global pb_start_of_program_flag ;
        if {$pb_start_of_program_flag==0} {
	     ;
	 } else {
	 	MOM_output_literal "$block_min"
	 	MOM_output_literal "$block_max"
	    }
	set status_block_create 1
 return ;
}

#=============================================================
proc MOM_start_of_path { } {
#=============================================================
#       
#
#############################################	 

  global mom_tool_name   mom_path_name   mom_operation_name

  global pb_start_of_program_flag
  if {$pb_start_of_program_flag == 0} {
     PB_start_of_program ;
     set pb_start_of_program_flag 1
   }

  global circleNumber ; set circleNumber 0; #     -  (  )
  MOM_force once M_spindle S X Y Z R F

  global mom_imported_path_status
  global mom_kin_arc_output_mode
  if {[info exists mom_imported_path_status]==1} {
     if {"YES"==[string toupper "$mom_imported_path_status"]} {
       set mom_warning_info "WARNING: =$mom_operation_name -   ( -   !)"
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
       set mom_kin_arc_output_mode  "QUADRANT"  ;   #QUADRANT  FULL_CIRCLE , LINEAR
      } else {
        set mom_kin_arc_output_mode  "FULL_CIRCLE"  ;   #QUADRANT  FULL_CIRCLE , LINEAR
       }
     MOM_reload_kinematics
   }

#############################################
}

#=============================================================
proc MOM_nurbs_move { } {
#=============================================================
#       
#=============================================================
  MOM_abort "\n\n\t:  Heidenhain 'TNC-360',  FCQV NC63\n\t    !.\n\n"
}

#=============================================================
proc MOM_from_move { } {
#=============================================================
  global mom_feed_rate mom_feed_rate_per_rev  mom_motion_type mom_kin_max_fpm mom_pos mom_prev_pos
  catch { COOLANT_SET ; CUTCOM_SET ; SPINDLE_SET ; RAPID_SET ;  MODES_SET ; } errMes; MOM_log_message "$errMes" ;
}
#=============================================================
proc MOM_initial_move { } {
#=============================================================
  	global mom_motion_type
  	global mom_pos mom_prev_pos

  		MOM_force Once X Y Z F

  	if {$mom_motion_type == "RAPID" } {
	if {$mom_pos(2) <= $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_xy
   	}

  	} elseif { $mom_motion_type == "CYCLE" } {
  		MOM_rapid_cycle
  	} else {
  		MOM_linear_move
  	}
}
#=============================================================
proc MOM_initial_move_prev { } {
#=============================================================
  	global mom_motion_type
  	global mom_pos mom_prev_pos

#  		MOM_force Once X Y Z F

  	if {$mom_motion_type == "RAPID" } {

	if {$mom_pos(2) <= $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy_prev
   		MOM_do_template rapid_cycle_z_prev
   	} else {
   		MOM_do_template rapid_cycle_z_prev
   		MOM_do_template rapid_cycle_xy_prev
   	}
  	} elseif { $mom_motion_type == "CYCLE" } {
    			MOM_do_template rapid_cycle_xy_prev
   			MOM_do_template rapid_cycle_z_prev

  	} else {
#  		MOM_do_template feedrate_tnc
	        MOM_do_template linear_move_prev
  	}
}
#=============================================================
proc MOM_first_move { } {
#=============================================================
  	global mom_feed_rate mom_feed_rate_per_rev mom_motion_type
  	global mom_kin_max_fpm mom_pos mom_prev_pos
        catch { COOLANT_SET ; CUTCOM_SET ; SPINDLE_SET ; RAPID_SET ;  MODES_SET ; } errMes ; MOM_log_message "$errMes" ;
	if {[info exists mom_kin_max_fpm] != 0} {
	if {$mom_feed_rate >= $mom_kin_max_fpm}  {MOM_rapid_move ; return}}
	if {$mom_motion_type == "RAPID"} {MOM_rapid_move} else {MOM_linear_move}

	if {$mom_pos(2) <= $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_xy
	}
}
#=============================================================
proc MOM_end_of_path { } {
#=============================================================
}

#=============================================================
proc MOM_end_of_program { } {
#=============================================================
   	global mom_machine_time
   	global line1  d_time  max_length
#############################################	 
    	MOM_do_template end_of_program
    	set line1 [format "; MACHINE TIME = %.2f min" $mom_machine_time ]
    	set tm1 [ expr int($mom_machine_time/60.0) ]
    	set tm2 [ expr ($mom_machine_time/60.0 - $tm1) * 60.0 ]
    	set str2 [format " ( %.0f hour : %.0f min )" $tm1 $tm2]
    	if {$tm1!=0} { append line1 $str2 ; }
#############################################
    	MOM_output_literal "$line1"
    	global prog_name
	MOM_output_literal "END PGM $prog_name MM"

#############################################	 
        global divide_flag  mom_seqnum  mom_warning_info
 	if {$divide_flag=="OFF"} {
           if { [info exists mom_seqnum] } {
           	if {$mom_seqnum>=$max_length} {
           	  set tmp1 [ format "%.0f" $mom_seqnum ]
      		  set mom_warning_info "\nWARNING:    = $tmp1 !!! \n\t   Heidenhain 'TNC-360',  FCQV NC63\n"
      		  append mom_warning_info "\t   -       4000  (    ),~64 \n"
                  append mom_warning_info "\t     4000 -           .\n"
                  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
                  ;#MOM_abort "\n$mom_warning_info\n"
                  ; #    ,    x  4000
                  global  mom_event_handler_file_name
                  set strDir [file dirname $mom_event_handler_file_name]
                  set strMes [file join $strDir "mesnumtcl.tcl" ]
                  if {[file exists $strMes ]} {
                   ; # source $strMes
                   set my_wish "[MOM_ask_env_var UGII_CAM_AUXILIARY_DIR]ugwish.exe"
                   exec $my_wish $strMes -- $mom_warning_info
                   }
                  ;#
           	}
           }
	}
#############################################

  	LIST_FILE_TRAILER ; # Close file list
  	CLOSE_files ; # Close output file
}

#=============================================================
proc MOM_set_modes { } {
#=============================================================
  	MODES_SET
}

#=============================================================
proc MOM_first_tool { } {
#=============================================================
}

#=============================================================
proc MOM_load_tool { } {
#=============================================================
}

#############################################	 
#=============================================================
proc  MOM_tool_change {} {
#      
#=============================================================
  global mom_tool_name
  global status_tool_change ; set status_tool_change 0
}
#############################################

#=============================================================
proc MOM_msys { } {
#=============================================================
	global mom_spindle_speed
	global status_block_create mom_tolerance_on_off

	if {$status_block_create == "0"} { MOM_block_create }
	MOM_spindle_rpm
}

#=============================================================
proc MOM_spindle_rpm { } {
#=============================================================
	global mom_tool_number  mom_tool_name   mom_tool_diameter
	global mom_spindle_speed
	global mom_operation_name
	global line
	global status_tool_change  spindle_off_flag

#############################################	 
 if {$mom_tool_number == 0} {
   MOM_abort "\n\n!Error!:\n\t      $mom_tool_number   $mom_operation_name\n\n"
  }

 if {[info exists mom_tool_diameter]==0} { set mom_tool_diameter 0.0 ; }
 if {[info exists mom_tool_name]==0} { set mom_tool_name "D0" ; }

	if {$status_tool_change == 0} {

   	set line [format "TOOL CALL %d Z S%.0f ; TOOL NAME = %s RADIUS = %.2f" $mom_tool_number $mom_spindle_speed $mom_tool_name [expr $mom_tool_diameter/2]]
   	MOM_output_literal "$line"
                                      
   		if {$spindle_off_flag == 0}	{
			if {$mom_spindle_speed == 0.0} {
				MOM_abort "      0   $mom_operation_name"
			}			
   			MOM_do_template spindle_rpm
   			set spindle_off_flag 1
		}
   		set status_tool_change 1
   	} else {
   		if {$spindle_off_flag == 0}	{
	                MOM_force once M_spindle F	
   			MOM_do_template spindle_rpm
   			set spindle_off_flag 1
		}
      }  
  
  
#############################################
}

#=============================================================
proc MOM_spindle_off { } {
#=============================================================
  global spindle_off_flag
  MOM_do_template spindle_off
  set spindle_off_flag 0
}
#=============================================================
proc MOM_coolant_on { } {
#=============================================================
  	COOLANT_SET
    	MOM_do_template coolant_on
}
#=============================================================
proc MOM_coolant_off { } {
#=============================================================
  	COOLANT_SET
    	MOM_do_template coolant_off
}
#=============================================================
proc MOM_cutcom_on { } {
#=============================================================
	global mom_cutcom_mode mom_cutcom_status
	global cutcom_mode  cutcom_on

	set cutcom_mode "R"
   	if {$mom_cutcom_mode == "OFF"} {
      		set cutcom_mode "0"
   	} elseif {$mom_cutcom_mode == "RIGHT"} {
      		set cutcom_mode "R"
   	} elseif {$mom_cutcom_mode == "LEFT"} {
      		set cutcom_mode "L"
   	} elseif {$mom_cutcom_mode == "ON"} {
      		set cutcom_mode "R"
   	}
   	if {$mom_cutcom_status == "OFF"} { set cutcom_mode "0" }
   	set cutcom_on 1
}
#=============================================================
proc MOM_cutcom_off { } {
#=============================================================
	global cutcom_mode  cutcom_on
	set cutcom_mode "0" ; set cutcom_on 0
	MOM_force once R
}
#=============================================================
proc MOM_linear_move { } {
#=============================================================
	global mom_cutcom_mode mom_cutcom_status
	global mom_pos mom_prev_pos
	global mom_smooth_path mom_small_steps  cutcom_mode  cutcom_on
	global mom_motion_distance mom_tool_diameter

	if {$mom_small_steps == "ON" 		&&
		$mom_cutcom_status == "ON"	&&
		$mom_motion_distance <= $mom_tool_diameter} {
			MOM_do_template feedrate_tnc
		   	MOM_do_template linear_move_m97
	} elseif {$mom_smooth_path == "ON"} {
		if {$cutcom_on == 1} {
			MOM_do_template linear_move
			set cutcom_on 0
		} else {
			MOM_do_template feedrate_tnc
			MOM_do_template linear_move_m90
		}
	} else  {
		MOM_do_template linear_move
	}

}

#=============================================================
proc MOM_circular_move { } {
#=============================================================
  global mom_arc_direction  mom_arc_axis   mom_arc_center
#############################################	 
    set xc  $mom_arc_center(0)
    set yc  $mom_arc_center(1)
    set zc  $mom_arc_center(2)
  global xc_Prev yc_Prev zc_Prev
  global circleNumber ; incr circleNumber ;
  set var_variant_circle 1 ; # 1 -   .      ;   -   .  
  if {$var_variant_circle==1} {

   if {$circleNumber!=1} {
	if {[format "%.6f" $mom_arc_axis(2)] == "1" || [format "%.6f" $mom_arc_axis(2)] == "-1"} {
     	  set fl 0
     	  MOM_suppress once ZC ; MOM_force once  XC YC ;
     	  if {[EQ_is_equal $xc $xc_Prev]} { incr fl }
     	  if {[EQ_is_equal $yc $yc_Prev]} { incr fl }
     	  if {[EQ_is_equal $fl 2]} { MOM_suppress once  XC YC ; }
	} elseif {[format "%.6f" $mom_arc_axis(1)] == "1" || [format "%.6f" $mom_arc_axis(1)] == "-1"} {
     	  	set fl 0
          	MOM_suppress once YC ; MOM_force once  XC ZC ;
     	  	if {[EQ_is_equal $xc $xc_Prev]} { incr fl ; }
     	  	if {[EQ_is_equal $zc $zc_Prev]} { incr fl ; }
     	  	if {[EQ_is_equal $fl 2]} { MOM_suppress once  XC ZC ; }
	} elseif {[format "%.6f" $mom_arc_axis(0)] == "1" || [format "%.6f" $mom_arc_axis(0)] == "-1"} {
          		set fl 0
     	  		MOM_suppress once XC ; MOM_force once  YC ZC ;
     	  		if {[EQ_is_equal $yc $yc_Prev]} { incr fl ; }
     	  		if {[EQ_is_equal $zc $zc_Prev]} { incr fl ; }
     	  		if {[EQ_is_equal $fl 2]} { MOM_suppress once  YC ZC; }
	}
    } else {
	if {[format "%.6f" $mom_arc_axis(2)] == "1" || [format "%.6f" $mom_arc_axis(2)] == "-1"} {
     	  MOM_suppress once ZC ; MOM_force once  XC YC ;
	} elseif {[format "%.6f" $mom_arc_axis(1)] == "1" || [format "%.6f" $mom_arc_axis(1)] == "-1"} {
          	MOM_suppress once YC ; MOM_force once  XC ZC ;
	} elseif {[format "%.6f" $mom_arc_axis(0)] == "1" || [format "%.6f" $mom_arc_axis(0)] == "-1"} {
     	  		MOM_suppress once XC ; MOM_force once  YC ZC ;
	}
      }

  } else {
	if {[format "%.6f" $mom_arc_axis(2)] == "1" || [format "%.6f" $mom_arc_axis(2)] == "-1"} {
		MOM_force once XC YC
		MOM_suppress once ZC
	} elseif {[format "%.6f" $mom_arc_axis(1)] == "1" || [format "%.6f" $mom_arc_axis(1)] == "-1"} {
		 MOM_force once XC ZC
		 MOM_suppress once YC
	} elseif {[format "%.6f" $mom_arc_axis(0)] == "1" || [format "%.6f" $mom_arc_axis(0)] == "-1"} {
		 MOM_force once YC ZC
		 MOM_suppress once XC
	}
}
#############################################
;#        MOM_do_template feedrate_tnc
   	MOM_do_template circle_center
   	if {[format "%.6f" $mom_arc_axis(2)] == "1" || [format "%.6f" $mom_arc_axis(2)] == "-1"} {
		 MOM_force once X Y
		MOM_suppress once Z
	} elseif {[format "%.6f" $mom_arc_axis(1)] == "1" || [format "%.6f" $mom_arc_axis(1)] == "-1"} {
		 MOM_force once X Z
		 MOM_suppress once Y
	} elseif {[format "%.6f" $mom_arc_axis(0)] == "1" || [format "%.6f" $mom_arc_axis(0)] == "-1"} {
		 MOM_force once Y Z
		 MOM_suppress once X
	}
   	if { $mom_arc_direction == "CLW"} {
      		MOM_do_template circle_end_cw
   	} elseif { $mom_arc_direction == "CCLW" } {
     		MOM_do_template circle_end_ccw
   	}

    set xc_Prev  $mom_arc_center(0)
    set yc_Prev  $mom_arc_center(1)
    set zc_Prev  $mom_arc_center(2)
  return
}

#=============================================================
proc MOM_rapid_move { } {
#=============================================================
	MOM_do_template rapid_move
}


############################################################################################

#=============================================================
proc MOM_rapid_cycle { } {
#=============================================================
	global mom_pos  mom_prev_pos
	if {$mom_pos(2) <= $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_xy
   	}
}

#=============================================================
proc MOM_rapid_move_cycle { } {
#=============================================================
   	global mom_pos mom_prev_pos mom_cycle_rapid_to mom_cycle_rapid_to_pos

   	set mom_pos(2) $mom_cycle_rapid_to_pos(2)
  	if {$mom_pos(2) <= $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_xy
   	}
}

#==================================================================
proc MOM_cycle_plane_change {} {
#==================================================================
# This procedure is executed at the end of cycle motion when there is a cycle
# move occuring at a higher level than the previous position.
   MOM_rapid_move_cycle
}

#==================================================================
proc MOM_cycle_off {} {
#==================================================================
# This procedure is executed at the end of cycle motion.
}

#=============================================================
proc MOM_drill { } {
#=============================================================
  	global mom_cycle_delay_mode
  	global mom_cycle_feed_to  mom_cycle_rapid_to_pos

  	global surface_coordinate ; set surface_coordinate "0.0"
  	MOM_do_template cycle_1_0 ; # DEF
  	MOM_do_template cycle_1_1 ; # SET UP
  	MOM_do_template cycle_1_2 ; # DEPTH
  	MOM_do_template cycle_1_3 ; # PECKG
; #  	MOM_do_template cycle_1_4 ; # DWELL
  	MOM_do_template cycle_1_5 ; # F
  	; # set mom_cycle_feed_to [expr abs($mom_cycle_feed_to)]
}

#=============================================================
proc  MOM_drill_move { } {
#=============================================================
  MOM_rapid_move_cycle
  MOM_do_template cycle_call
}

#=============================================================
proc MOM_drill_dwell { } {
#=============================================================
  	global mom_cycle_delay_mode  mom_cycle_delay  mom_cycle_delay_revs
  	global mom_cycle_feed_to

  	global surface_coordinate ; set surface_coordinate "0.0"
  	global dwell_at_top ;	set dwell_at_top "0.0"
  	if {$mom_cycle_delay_mode != "OFF"} { set dwell_at_top $mom_cycle_delay ; }
    	MOM_do_template cycle_1_0 ; # DEF
  	MOM_do_template cycle_1_1 ; # SET UP
  	MOM_do_template cycle_1_2 ; # DEPTH
 ;#  	MOM_do_template cycle_1_3 ; # PECKG
  	MOM_do_template cycle_1_4 ; # DWELL
  	MOM_do_template cycle_1_5 ; # F
 ;#  	set mom_cycle_feed_to [expr abs($mom_cycle_feed_to)]
}

#=============================================================
proc  MOM_drill_dwell_move { } {
#=============================================================
  MOM_rapid_move_cycle
  MOM_do_template cycle_call
}

#=============================================================
proc MOM_drill_deep { } {
#=============================================================
  	global mom_cycle_delay_mode mom_cycle_delay
  	global mom_cycle_feed_to
  	global mom_cycle_cam ; 	set mom_cycle_cam "0"

  	global surface_coordinate ; set surface_coordinate "0.0"
  	global dwell_at_top ; set dwell_at_top "0.0"
  	if {$mom_cycle_delay_mode != "OFF"} { set dwell_at_top $mom_cycle_delay ; }
  	MOM_do_template cycle_1_0 ; # DEF
  	MOM_do_template cycle_1_1 ; # SET UP
  	MOM_do_template cycle_1_2 ; # DEPTH
  	MOM_do_template cycle_1_3 ; # PECKG
  	MOM_do_template cycle_1_4 ; # DWELL
  	MOM_do_template cycle_1_5 ; # F
	set mom_cycle_cam $mom_cycle_cam
}
#=============================================================
proc  MOM_drill_deep_move { } {
#=============================================================
  MOM_rapid_move_cycle
  MOM_do_template cycle_call
}
#=============================================================
proc MOM_drill_break_chip { } {
#=============================================================
  	global mom_cycle_delay_mode mom_cycle_delay
  	global mom_cycle_feed_to

  	global surface_coordinate ; set surface_coordinate "0.0"
  	global dwell_at_top ; set dwell_at_top "0.0"
  	if {$mom_cycle_delay_mode != "OFF"} { set dwell_at_top $mom_cycle_delay ; }

  	MOM_do_template cycle_1_0 ; # DEF
  	MOM_do_template cycle_1_1 ; # SET UP
  	MOM_do_template cycle_1_2 ; # DEPTH
  	MOM_do_template cycle_1_3 ; # PECKG
  	MOM_do_template cycle_1_4 ; # DWELL
  	MOM_do_template cycle_1_5 ; # F

}
#=============================================================
proc  MOM_drill_break_chip_move { } {
#=============================================================
  MOM_rapid_move_cycle
  MOM_do_template cycle_call
}
#=============================================================
proc MOM_tap { } {
#=============================================================
	MOM_do_template cycle_17_1
	MOM_do_template cycle_17_2
	MOM_do_template cycle_17_3
	MOM_do_template cycle_17_4
}
#=============================================================
proc  MOM_tap_move { } {
#=============================================================
   MOM_rapid_move_cycle
   MOM_do_template cycle_call
}
#=============================================================
proc MOM_bore { } {
#=============================================================
  global mom_warning_info
  set mom_warning_info "\nWARNING:       \[bore\]\n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}
#=============================================================
proc  MOM_bore_move { } {
#=============================================================
   MOM_linear_move
}
#=============================================================
proc MOM_bore_drag { } {
#=============================================================
  global mom_warning_info
  set mom_warning_info "\nWARNING:       \[bore_drag\]\n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}
#=============================================================
proc  MOM_bore_drag_move { } {
#=============================================================
    MOM_linear_move
}
#=============================================================
proc MOM_bore_no_drag { } {
#=============================================================
  global mom_warning_info
  set mom_warning_info "\nWARNING:   C   \[bore_no_drag\]    \n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}
#=============================================================
proc  MOM_bore_no_drag_move { } {
#=============================================================
    MOM_linear_move
}
#=============================================================
proc MOM_bore_manual { } {
#=============================================================
  global mom_warning_info
  set mom_warning_info "\nWARNING:   C   \[bore_manual\]    \n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}
#=============================================================
proc  MOM_bore_manual_move { } {
#=============================================================
	MOM_linear_move
}
#=============================================================
proc MOM_bore_dwell { } {
#=============================================================
  global mom_warning_info
  set mom_warning_info "\nWARNING:   C  \[bore_dwell\]    \n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}
#=============================================================
proc  MOM_bore_dwell_move { } {
#=============================================================
	MOM_linear_move
}
#=============================================================
proc MOM_bore_back { } {
#=============================================================
  global mom_warning_info
  set mom_warning_info "\nWARNING:    \[bore_back\]    \n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}
#=============================================================
proc  MOM_bore_back_move { } {
#=============================================================
	MOM_linear_move

}
#=============================================================
proc MOM_bore_manual_dwell { } {
#=============================================================
  global mom_warning_info
  set mom_warning_info "\nWARNING:   C     \[bore_manual_dwell\]    \n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}
#=============================================================
proc  MOM_bore_manual_dwell_move { } {
#=============================================================
	MOM_linear_move
}

############################################################################################

#=============================================================
proc MOM_sequence_number { } {
#=============================================================
}
#=============================================================
proc PB_CMD_before_motion { } {
#=============================================================
}

#=============================================================
proc PB_approach_move { } {
#=============================================================
}
#=============================================================
proc PB_engage_move { } {
#=============================================================
}
#=============================================================
proc PB_first_cut { } {
#=============================================================
}
#=============================================================
proc PB_return_move { } {
#=============================================================
}
#=============================================================
proc MOM_gohome_move { } {
#=============================================================
	global mom_pos  mom_prev_pos  mom_mcs_goto
	set mom_pos(0)	$mom_mcs_goto(0)
	set mom_pos(1)	$mom_mcs_goto(1)
	set mom_pos(2)	$mom_mcs_goto(2)
	set mom_pos(3)	0.0
	if {$mom_pos(2) < $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_xy
   	}
}

#=============================================================
proc MOM_machine_mode {} {
#=============================================================
	MOM_start_of_group
}
#=============================================================
proc MOM_delay { } {
#=============================================================
  	global mom_sys_delay_param mom_delay_value
  	global mom_delay_revs mom_delay_mode delay_time
  	global mom_warning_info

        # post builder provided format for the current mode:
  	if {[info exists mom_sys_delay_param(${mom_delay_mode},FORMAT)] != 0} {
    	 MOM_set_address_format dwell $mom_sys_delay_param(${mom_delay_mode},FORMAT)
  	}

  	switch $mom_delay_mode {
    	 SECONDS {set delay_time $mom_delay_value}
    	 default {
    	 	  set delay_time $mom_delay_revs ;
                  set mom_warning_info "\nWARNING:      . !\n"
         	  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
    	 	}
  	}
    	MOM_do_template delay_1
    	MOM_do_template delay_2
}
#=============================================================
proc MOM_opstop { } {
#=============================================================
	MOM_do_template opstop
}

#==================================================================
proc MOM_opskip_on {} {
#==================================================================
# This procedure is executed when the Optional skip command is activated.
  global  mom_sys_opskip_block_leader
  MOM_set_line_leader always "$mom_sys_opskip_block_leader"
}

#==================================================================
proc MOM_opskip_off {} {
#==================================================================
# This procedure is executed when the Optional skip command is activated.
  global  mom_sys_opskip_block_leader
  MOM_set_line_leader off "$mom_sys_opskip_block_leader"
}

#=============================================================
proc MOM_auxfun { } {
#=============================================================
	MOM_do_template auxfun
}
#=============================================================
proc MOM_prefun { } {
#=============================================================
	MOM_do_template prefun
}
#=============================================================
proc MOM_stop { } {
#=============================================================
	MOM_do_template stop
}

#==================================================================
proc  MOM_zero {} {
#==================================================================
# This procedure is executed when the ZERO command is activated.
# G53 P_ ; G54 X_ Y_ Z_ ; G54 X0 Y0 Z0
   global mom_warning_info
   global mom_imported_path_status
   if {[info exists mom_imported_path_status]==1} {
     if {"YES"==[string toupper "$mom_imported_path_status"]} {
       set mom_warning_info "WARNING:       CYCL DEF 7.0 DATUM SHIFT "
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
       return ;
      }
   }

  set mom_warning_info "WARNING:       CYCL DEF 7.0 DATUM SHIFT "
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
  return ;
}


#==================================================================
proc MOM_path_warning { } {
#==================================================================
# This procedure is executed when the in CLSF file has Inconsistent axis.
  global mom_seqnum
  global mom_warning_info
  global mom_path_warning
  if {[info exists mom_path_warning]} {
     if {[string length $mom_path_warning]!=0} {
     	set tmpNum [format "%.0f" $mom_seqnum]
        set mom_warning_info "!! :     ( ?     ) = $mom_path_warning ( =$tmpNum)"
     	MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
     	}
   }
  #MOM_abort $mom_warning_info
}

#############################################################################################
#=============================================================
proc MOM_small_step_contour { } {
#=============================================================
	global mom_small_steps

}

#=============================================================
proc MOM_smooth_tool_path { } {
#=============================================================
	global mom_smooth_path

}

#=============================================================
proc MOM_rotation_cycle { } {
#=============================================================
   global mom_rotation mom_rotation_angle

   set mom_warning_info "\nWARNING:  CYCL DEF 10.0 ROTATION  -    () !\n"
   MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

#=============================================================
proc MOM_datum_shift_cycle { } {
#=============================================================
   global mom_datum_shift

   set mom_warning_info "\nWARNING:  CYCL DEF 7.0 DATUM SHIFT  -    () !\n"
   MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

#=============================================================
proc MOM_scaling_cycle { } {
#=============================================================
   global mom_scaling  mom_scaling_factor

   set mom_warning_info "\nWARNING:  CYCL DEF 11.0 SCALING  -    () !\n"
   MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

#=============================================================
proc MOM_tolerance_cycle { } {
#=============================================================
	global mom_tolerance_on_off  tolerances
	global mom_operation_type  mom_inside_outside_tolerances

  	if {$mom_operation_type == "Variable-axis Surface Contouring"  	||
		$mom_operation_type == "Cavity Milling" ||
		$mom_operation_type == "Fixed-axis Surface Contouring"  ||
		$mom_operation_type == "Planar Milling"  ||
		$mom_operation_type == "Sequential Mill Main Operation"  ||
		$mom_operation_type == "Zig-Zag Surface" } {
  		set temp [expr $mom_inside_outside_tolerances(0) + $mom_inside_outside_tolerances(1)]
  		set tolerances [expr $temp/2]
  	}
   set mom_warning_info "\nWARNING:  CYCL DEF 32.0 TOLERANCE  TNC 360 -   !\n"
   MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

#=============================================================
proc MOM_spindle_orient { } {
#=============================================================
	global mom_angle_orient
	MOM_force once Angle
	MOM_do_template spindle_orient_1
	MOM_do_template spindle_orient_2
}

#=============================================================
proc  MOM_divide { } {
#=============================================================
# "DIVIDE"
        global mom_command_status
	global mom_divide_by mom_length_of_program mom_live_time
	global mom_retract_to mom_offset_dist mom_retract_point_pos
	global max_length
	set max_length [expr double($mom_length_of_program)]
global divide_flag ; set divide_flag ON
}

#=============================================================
proc  MOM_before_motion { } {
#=============================================================
	global mom_motion_event mom_motion_type mom_o_buffer
	global length accom_length max_length
	global ptp_file_name mom_output_file_directory mom_output_file_basename
	global mom_operation_name
	global output_extn i
	global mom_divide_by mom_cutting_time mom_live_time
	global mom_retract_to mom_retract_point_pos mom_offset_dist
	global total_cutting_time accumulated_time accom_length length
	global mom_seqnum header_check_status status_block_create
	global mom_pos mom_prev_pos

	set accom_length $mom_seqnum

	FEEDRATE_SET
	switch $mom_motion_type {
    		ENGAGE {PB_engage_move}
    		APPROACH {PB_approach_move}
    		FIRSTCUT {PB_first_cut}
  	}
  	if {[llength [info commands PB_CMD_before_motion]]} {PB_CMD_before_motion}

#############################################	 
  global mom_imported_path_status
  if {[info exists mom_imported_path_status]==1} {
    if {"YES"==[string toupper "$mom_imported_path_status"]} {
     ; #
     if {[info exists mom_motion_event]==1} {
       if {$mom_motion_event!="rapid_move"} {
         global feed ;
         if {$feed>1500.0} {
           set feed 1500.0 ;
           set mom_warning_info "WARNING:        =$feed"
           MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
         }
       }
     }
     ; #
   }
 }
#############################################

	set total_cutting_time [expr $mom_cutting_time - $accumulated_time]

#############################################	 
  global divide_flag ;
  if {$divide_flag=="ON"} {

	if {$mom_divide_by == "LENGTH"} {
	  if { $accom_length >= $max_length } {
	    if { $mom_motion_type == "STEPOVER" || $mom_motion_type == "RETRACT" } {
		if {$mom_retract_to == "OFFSET"} {
		  MOM_do_template go_offset
		 } elseif {$mom_retract_to == "POINT"} {
		 		MOM_force Once X Y Z
				MOM_do_template go_point
			}
		MOM_force Once M
		MOM_end_of_program
  		MOM_close_output_file $ptp_file_name
#############################################	 
  	        ;#CLOSE_files ; # Close output file
  		global count ; incr count ; set str [format "%02d" $count] ;
  		global fileStr ;
	        set mom_output_file_basename "$fileStr$str"
		set ptp_file_name "${mom_output_file_directory}${mom_output_file_basename}${output_extn}"
		if {[file exists $ptp_file_name]} {
      		   MOM_remove_file $ptp_file_name
    		}
		MOM_open_output_file $ptp_file_name
		;#OPEN_files ; #   ( )  
#############################################
		set header_check_status 0
		set status_block_create 0
		MOM_reset_sequence 1 1 1
		MOM_force Once X Y Z F M_spindle
#############################################	 
		 MOM_start_of_group
 	         if { $header_check_status == 0} {
 	         	global prog_name
		   MOM_output_literal "BEGIN PGM $prog_name MM"
		   set header_check_status 1
	         }
#############################################
		MOM_msys

		if {$mom_retract_to == "OFFSET"} {

                   set  mom_prev_pos(2)  [ expr ($mom_prev_pos(2)+$mom_offset_dist)]
                  # MOM_initial_move
		   MOM_force Once X Y Z F
 		   MOM_initial_move_prev
                  #   Z
                   set  mom_prev_pos(2)  [ expr ($mom_prev_pos(2)-$mom_offset_dist)]
 		   MOM_initial_move_prev

		} elseif {$mom_retract_to == "POINT"} {
			MOM_force Once X Y Z
                        MOM_do_template go_point
 			MOM_initial_move_prev
			}
		set accom_length 0
	   }
      }

    } elseif {$mom_divide_by == "TIME"} {
	   if { $total_cutting_time >= $mom_live_time } {
	     if { $mom_motion_type == "STEPOVER" || $mom_motion_type == "RETRACT" } {
		if {$mom_retract_to == "OFFSET"} {
		   MOM_do_template go_offset
		 } elseif {$mom_retract_to == "POINT"} {
		 	MOM_force Once X Y Z
			MOM_do_template go_point
		    }
		MOM_force Once M
		MOM_end_of_program
  		MOM_close_output_file $ptp_file_name
#############################################	 
  	        ;#CLOSE_files ; # Close output file
  		global count ; incr count ; set str [format "%02d" $count] ;
  		global fileStr ;
	        set mom_output_file_basename "$fileStr$str"
		set ptp_file_name "${mom_output_file_directory}${mom_output_file_basename}${output_extn}"
		if {[file exists $ptp_file_name]} {
      			MOM_remove_file $ptp_file_name
    		}
		MOM_open_output_file $ptp_file_name
		;#OPEN_files ; #   ( )  
#############################################
		set header_check_status 0
		set status_block_create 0
		MOM_reset_sequence 1 1 1
		MOM_force Once X Y Z F M_spindle
#############################################	 
		 MOM_start_of_group
 	         if { $header_check_status == 0} {
 	         	global prog_name
		   MOM_output_literal "BEGIN PGM $prog_name MM"
		   set header_check_status 1
	         }
#############################################
		MOM_msys
               #MOM_initial_move
		if {$mom_retract_to == "OFFSET"} {
                   set  mom_prev_pos(2)  [ expr ($mom_prev_pos(2)+$mom_offset_dist)]
                   #MOM_initial_move
		   MOM_force Once X Y Z F
 		   MOM_initial_move_prev
                   #   Z
                   set  mom_prev_pos(2)  [ expr ($mom_prev_pos(2)-$mom_offset_dist)]
 		   MOM_initial_move_prev
		} elseif {$mom_retract_to == "POINT"} {
		 	MOM_force Once X Y Z
                        MOM_do_template go_point
 			MOM_initial_move_prev
		      }
		set total_cutting_time 0
		set accumulated_time $mom_cutting_time
    	}
     }
  }


 } ; # end if {$divide_flag=="ON"}
#############################################
  return ;
}
