##################################TCL FILE####################################
# Description
#     This is the event handler file for generic 3-axis Mill-Turn with
#     Heidenhain Control System  (HEIDENHAIN TNC 360 )
#
# System:  "HEIDENHAIN TNC-360 (iso + heidenhain code)"
# Machine: "FCQV NC63, ..."
# Type: MILL 2.5
# version xx.xx.xx (04.04.2003 14:14)
# Russia, KnAAPO,OGT,SPC(Sector of the Program Control,#"  ")
# Author : .
###############################################################################
#
# 22-23 --2002       0. Original
# 15.03.2003 9:38       Rodionova Elena      1.    ,  {0}  
#                                            2.   'Zero'
# 18.04.2003 16:23          1.    ,
#                                            2.        =
#                                                {  ( .    1500.0 +   ./   }
#                                            3.  'MOM_circular_move' -   .       
#  22.04.2003               1. o MOM_tolerance_cycle -   
#                                            2.  MOM_rotation_cycle ->    ()
#                                            3.  MOM_datum_shift_cycle ->    ()
#                                            4.  MOM_scaling_cycle ->    ()
#                                            6.  ( )  (  TNC 360   :
#                                                  G83(PECKING) G85(TAPPING) G85(RIGID TAPPING GS) , .  )
################################################################################
;# source "[MOM_ask_env_var UGII_CAM_DEBUG_DIR]mom_review.tcl" ; MOM_set_debug_mode ON

#_______________________________________________________________________________
#The following command invokes the warning mode and the default settings.
  source "[MOM_ask_env_var UGII_CAM_POST_DIR]ugpost_base.tcl"
#_______________________________________________________________________________
#The following file consists of default values set for G & M codes


########## SYSTEM VARIABLE DECLARATIONS ##############
  set mom_sys_rapid_code                        "0"
  set mom_sys_linear_code                       "1"
  set mom_sys_circle_code(CLW)                  "2"
  set mom_sys_circle_code(CCLW)                 "3"
  set mom_sys_delay_code(SECONDS)               "4"
  set mom_sys_delay_code(REVOLUTIONS)           "4"
  set mom_sys_cutcom_plane_code(XY)             "17"
  set mom_sys_cutcom_plane_code(ZX)             "18"
  set mom_sys_cutcom_plane_code(YZ)             "19"
  set mom_sys_cutcom_code(OFF)                  "40"
  set mom_sys_cutcom_code(LEFT)                 "41"
  set mom_sys_cutcom_code(RIGHT)                "42"
  set mom_sys_adjust_code                       "43"
  set mom_sys_adjust_code_minus                 "44"
  set mom_sys_adjust_cancel_code                "40"
  set mom_sys_unit_code(IN)                     "70"
  set mom_sys_unit_code(MM)                     "71"
#
  set mom_sys_cycle_breakchip_code              "83"
  set mom_sys_cycle_drill_code                  "83"
  set mom_sys_cycle_drill_deep_code             "83"
  set mom_sys_cycle_drill_dwell_code            "83"
  set mom_sys_cycle_tap_code                    "85"
  set mom_sys_cycle_start_code                  "99"
#
  set mom_sys_output_code(ABSOLUTE)             "90"
  set mom_sys_output_code(INCREMENTAL)          "91"
  set mom_sys_cycle_ret_code(AUTO)              "98"
  set mom_sys_cycle_ret_code(MANUAL)            "99"
  set mom_sys_reset_code                        "79"
#--
  set mom_sys_feedrate_code(IPM)        "94"
  set mom_sys_feedrate_code(MMPM)       "94"
  set mom_sys_feedrate_code(IPR)        "95"
  set mom_sys_feedrate_code(MMPR)       "95"
#--
  set mom_sys_feed_rate_mode_code(MMPM)         "94"
  set mom_sys_feed_rate_mode_code(MMPR)         "95"
  set mom_sys_feed_rate_mode_code(FRN)          "93"
  set mom_sys_spindle_mode_code(SFM)            "96"
  set mom_sys_spindle_mode_code(SFM)            "96"
  set mom_sys_spindle_mode_code(RPM)            "97"
  set mom_sys_return_code                       "28"
  set mom_sys_program_stop_code                 "0"
  set mom_sys_optional_stop_code                "1"
  set mom_sys_end_of_program_code               "2"
  set mom_sys_spindle_direction_code(NONE)      "3"
  set mom_sys_spindle_direction_code(CLW)       "3"
  set mom_sys_spindle_direction_code(CCLW)      "4"
  set mom_sys_spindle_direction_code(OFF)       "5"
  set mom_sys_tool_change_code                  "6"
  set mom_sys_coolant_code(ON)                  "8"
  set mom_sys_coolant_code(MIST)                "7"
  set mom_sys_coolant_code(FLOOD)               "8"
  set mom_sys_coolant_code(TAP)                 "8"
  set mom_sys_coolant_code(OFF)                 "9"
  set mom_sys_rewind_code                       "30"
  set mom_sys_spindle_range_code                "40"
  set mom_sys_spindle_ranges                    "0"
  set mom_sys_rewind_stop_code                  "%"
  set mom_sys_home_pos(0)                       "0"
  set mom_sys_home_pos(1)                       "0"
  set mom_sys_home_pos(2)                       "0"
  set mom_sys_zero                              "0"
  set mom_sys_opskip_block_leader               "/"
  set mom_sys_seqnum_start                      "1"
  set mom_sys_seqnum_incr                       "1"
  set mom_sys_seqnum_freq                       "1"
  set mom_sys_leader(N)                         "N"
  set mom_sys_leader(X)                         "X"
  set mom_sys_leader(Y)                         "Y"
  set mom_sys_leader(Z)                         "Z"
  set mom_sys_feed_param(MMPM,format)           "Feed_MMPM"
  set mom_sys_feed_param(FRN,format)            "Feed_INV"
  set mom_sys_control_out                       " ;"
  set mom_sys_control_in                        ""
####
  set mom_sys_wcs_change_code                53
  set mom_work_coordinate_number              0
  set mom_sys_mirror_image_on                28
  set mom_sys_mirror_image_off               28
####
####### KINEMATIC VARIABLE DECLARATIONS ##############
  set mom_kin_coordinate_type                   "CARTESIAN "
  set mom_kin_arc_output_mode                   "FULL_CIRCLE"  ;   #QUADRANT  FULL_CIRCLE , LINEAR
  set mom_kin_nurbs_output_type			"HEIDENHAIN_POLY"
  set mom_kin_arc_valid_planes                  "XYZ"  ;  # "XYZ" "ANY"
;#  set mom_kin_arc_valid_plane                   "XY"
  set mom_kin_clamp_time                        "2.0"
  set mom_kin_linearization_flag                "F"
  set mom_kin_linearization_tol                 "0.05"
  set mom_kin_machine_resolution                "0.001"
  set mom_kin_machine_type                      "3_axis_mill"
  set mom_kin_max_arc_radius                    "30000.000"
  set mom_kin_max_fpm                           "8000"
  set mom_kin_max_frn                           "99999.999"
  set mom_kin_min_arc_length                    "0.025"
  set mom_kin_min_arc_radius                    "0.016"
  set mom_kin_min_fpm                           "1.0"
  set mom_kin_min_frn                           "0.001"
  set mom_kin_output_unit                       "MM"
  set mom_kin_pivot_gauge_offset                "0.0"
  set mom_kin_post_data_unit                    "MM"
  set mom_kin_rapid_feed_rate                   "99999.99"
  set mom_kin_tool_change_time                  "12.0"
  set mom_kin_x_axis_limit                      "1500" ; # "900"
  set mom_kin_y_axis_limit                      "1000" ; # "300"
  set mom_kin_z_axis_limit                      "500"  ; # "120"
  set mom_kin_read_ahead_next_motion            "F"

####  Listing File variables
  set mom_sys_list_output                       "ON"
  set mom_sys_header_output                     "ON"
  set mom_sys_list_file_rows                    "120"
  set mom_sys_list_file_columns                 "150"
  set mom_sys_warning_output                    "OFF"
  set mom_sys_group_output                      "OFF"
  set mom_sys_list_file_suffix                  "lpt"
  set mom_list_file_suffix              	"lpt"
  set mom_sys_output_file_suffix                "i"
  set mom_output_file_suffix                	"i"
  set mom_sys_commentary_output                 "ON"
  set mom_sys_commentary_list                   "x y z feed speed"
############-----------------------------------------------------
  set mom_event_time                            "0.0"

#----------------------------- VARIABLES
  set initial_move 0
  set output_mode                           "CARTESIAN"
#      
set header_check_status						0
set mom_smooth_path						"OFF"
set mom_tolerance_on_off					"OFF"
set mom_small_steps						"OFF"
set status_block_create						0
set status_tool_change					      	0
set spindle_off_flag 						0
#
set mom_point1_pos(0) 						-100.0
set mom_point1_pos(1) 						-100.0
set mom_point1_pos(2) 						-50.0
set mom_point2_pos(0) 						100.0
set mom_point2_pos(1) 						100.0
set mom_point2_pos(2) 						0.0
set mom_offset_block						10.0
#
set fl_accom_length						0
set length 							0
set accom_length						0
set max_length							3900 ; #   
set i								1
set mom_divide_by						"LENGTH"
set accumulated_time						0
set mom_retract_to						"OFFSET"
set mom_offset_dist						100.0
set d_time							0
set cutcom_on 							0
set divide_flag                                                 OFF
#
set mom_time                    OFF ; #  
set mom_rewind_end_of_program     0 ; #  M30
set var_start_of_path_num         0 ; #   
set first_tool                    0 ; #  1- 
set type_postprocessors         "ISO"


#####################################################################
#==================================================================
proc MOM_msys {} {
#==================================================================
## This procedure is called for a MSYS statement
}


#==================================================================
proc  MOM_before_output {} {
#==================================================================
# This procedure is executed just before a line is about to be output
# to the file. It loads the line into a variable MOM_o_buffer and then calls
# this procedure. When it returns from this procedure, the variable MOM_o_buffer
# is read and written to the file.
#########The following procedure invokes the listing file with warnings.
 global mom_o_buffer
         LIST_FILE

 global accom_length  fl_accom_length
 global mom_seqnum

 set str $mom_o_buffer
 set var0 ""
 ;#    
 set st [regsub -nocase -- {(;)+(.)*} $str  ""  var0 ]
 set str  $var0

 #    N------
   set st [regexp -nocase -- {(N)([0-9])+}  $str  var0 ]
   if {$st!=0} {
      incr accom_length
   } ; #    N ----------

}


#=============================================================
proc PB_CMD_machine_mode { } {
#=============================================================
  global mom_output_mode mom_arc_mode mom_prev_machine_mode mom_machine_mode mom_feed_set_mode
  set mom_output_mode     	"ABSOLUTE"
  set mom_prev_machine_mode  	"MILL"
  set mom_machine_mode 		"MILL" ;
  set mom_arc_mode     		"CIRCULAR" ;
  set mom_feed_set_mode 	"MMPR" ;
}


#=============================================================
proc MOM_machine_mode {} {
#=============================================================
  catch {PB_CMD_machine_mode} strErr ; MOM_log_message "event MOM_machine_mode - $strErr"
  global mom_load_event_handler
  catch { MOM_log_message "event MOM_machine_mode->mom_load_event_handler=$mom_load_event_handler" }
}


#==================================================================
proc MOM_from_move {} {
#==================================================================
# This procedure is called for a FROM statement
  catch { COOLANT_SET ; CUTCOM_SET ; SPINDLE_SET ; RAPID_SET ; } strErr ; MOM_log_message "event MOM_from_move - $strErr"
  outDec on
  catch { MOM_do_template from CREATE } strErr ; MOM_log_message "event MOM_from_move - $strErr"
  outDec off
}


#==================================================================
proc MOM_gohome_move {} {
#==================================================================
# This procedure is called for a GOHOME move
     MOM_rapid_move ; #MOM_do_template gohome
}


#==================================================================
proc MOM_start_of_program {} {
#==================================================================
# This procedure is executed at the very begining of the program.
# It gets called before any command is read from the task file.
#
  global mom_coolant_mode   mom_coolant_status
  global mom_cutcom_mode   mom_cutcom_status

#---- Unsetting defult setting for variables
  catch {unset mom_coolant_mode} ;
  catch {unset mom_coolant_status}
  #catch {unset mom_cutcom_mode}  ;
  catch {unset mom_cutcom_status}

  PB_start

  OPEN_files ; #------- The following procedure opens the warning and listing files
  LIST_FILE_HEADER   ; #------- The following procedure lists the header information in commentary data

}

set Num  "" ;
#==================================================================
proc PB_start { } {
#==================================================================

  global mom_logname mom_date mom_ug_version mom_part_name
  global mom_event_handler_file_name   mom_definition_file_name
  global mom_event_handler_name
  global mom_load_event_handler
  global mom_output_file_directory  mom_output_file_basename  mom_output_file_suffix

  global pb_start_of_program_flag ;  set pb_start_of_program_flag 0

#------- These are to be obtained for the user before Postprocessing

 global Num
; #   
     set var0 ""
     set fl1 [regexp -- {([0-9])+} $mom_output_file_basename var0]
     if {$fl1=="0"} {
 	MOM_abort "\n\n\tWARNING: ( Heidenhain 'TNC-360',  FCQV-63NC)\n\t :   -   - ,   1  (max 8)\n\n"
     }
     set var1 ""
     regsub -nocase -- {(0)*} $var0 "" var1 ;
     if {$var1==""} {
 	MOM_abort "\n\n\tWARNING: ( Heidenhain 'TNC-360',  FCQV-63NC)\n\t :   -   - ,   1  (max 8)\n\n"
     }
     set Num [format "%s" $var1]

 ;#       
 if {[info exists mom_output_file_suffix]} {
     if {"H"==[string toupper $mom_output_file_suffix]} {
       set fileDef "tnc360_3_axis_h.def" ; set fileTcl "tnc360_3_axis_h.tcl" ;
       set strDir [file dirname $mom_definition_file_name]
       set strDef [file join $strDir $fileDef ]
       set fl 0;  if {[file exists $strDef]} { incr fl ; }
       set strDir [file dirname $mom_event_handler_file_name]
       set strTcl [file join $strDir $fileTcl ]
       if {[file exists $strTcl]} { incr fl ; }
       MOM_log_message "    (fl)=$fl"
       if {$fl==2} {
       	  global type_postprocessors ; set type_postprocessors "ISO" ;
       	  set fl 0 ; set errMes "" ;
       	  ;#
          ;# catch { source "$strTcl" ; } errMes ;
          set mom_event_handler_file_name  "$strTcl"
          set mom_event_handler_name  "$strTcl"
          set mom_load_event_handler "$strTcl"
          ;#
          if {$errMes==""} {
           MOM_log_message "   (Heidenhain format) = $strTcl"
           incr fl ; set errMes "" ;
           global tcl_platform
           if {$tcl_platform(os) == "HP-UX"} {
	    } elseif {$tcl_platform(os) == "windows" || $tcl_platform(os) == "Windows NT"} {
	    	      set var0 ""
		      regsub -all -nocase -- {(/)} $strDef "\\" var0
		      set strDef $var0 ;
	          } else { ; }
           catch { MOM_load_definition_file "$strDef" ; } errMes ;
           if {$errMes==""} {
            MOM_log_message "   (Heidenhain format)= $strDef"
            incr fl ;
           } else { MOM_abort "\n\n  \n:       $mom_output_file_suffix\n  \n\n"; }
          } else { MOM_abort "\n\n  \n:       $mom_output_file_suffix\n  \n\n"; }
         if {$fl==2} { set type_postprocessors  "HEIDENHAIN" }
         ;#  
          ;# MOM_load_definition_file "Z:\\postprocessor\\tnc360_3_axis_h.def" ;
          ;# source "Z:\\postprocessor\\tnc360_3_axis_h.tcl" ;
          ;#
        }
     } else {
     	set ln [string length $mom_output_file_suffix ]
 	set var0 ""
	regsub -all -nocase -- {(/)} $mom_output_file_suffix "" var0 ; 	set mom_output_file_suffix $var0 ;
	regsub -all -nocase -- {(\\)} $mom_output_file_suffix "" var0 ;	set mom_output_file_suffix $var0 ;
     	switch [string toupper $mom_output_file_suffix ] {
     	  "H" { ; } "I" { ; } "UP" { ; } "PTP" { ; } "DAT" { ; } "TXT" { ; }
     	 default { set mom_output_file_suffix "I" ; }
     	}
      }
 } else { set mom_output_file_suffix "I" }
 ;#

  MOM_log_message "mom_output_file_directory=$mom_output_file_directory \n mom_output_file_basename=$mom_output_file_basename \n mom_output_file_suffix=$mom_output_file_suffix"

}

#----------------------------------------------------
proc outDec { mode } {
#----------------------------------------------------
    switch $mode {
	on {
		global mom_pos
		for { set i 0 } {$i<3} {incr i} {
		     ;#1
		     if {[info exists mom_pos($i)]} {
			set str_real $mom_pos($i)
			set str_int [ expr int($str_real) ]
			if {[EQ_is_equal $str_real $str_int]} {
		 		switch $i {
		 		  0 { MOM_set_address_format X CoordinateDec }
		 		  1 { MOM_set_address_format Y CoordinateDec }
		 		  2 { MOM_set_address_format Z CoordinateDec }
		 		}
		 	 }
		     }
		     ;#2
		     global mom_motion_event	 mom_current_motion
		     if {[info exists mom_motion_event]} {
		    	if {$mom_motion_event == "circular_move"} {
			  global mom_pos_arc_center mom_prev_pos
		 	  ;#3
	 		  global mom_output_mode
      		  	  switch $mom_output_mode {
       			    ABSOLUTE {
       				set str_real $mom_pos_arc_center($i)
		 		set str_int [ expr int($str_real) ]
			     }
       			    default {
       				set str_real [ expr  $mom_pos_arc_center($i)-$mom_prev_pos($i) ]
		 		set str_int [ expr int($str_real) ]
     			     }
       		           }
       		          if {[EQ_is_equal $str_real $str_int]} {
		 		switch $i {
		 		  0 { MOM_set_address_format I CoordinateDec }
		 		  1 { MOM_set_address_format J CoordinateDec }
		 		  2 { MOM_set_address_format K CoordinateDec }
		 		}
		 	   }
      		   	 ;#3
		       }
		      }
	     	     ;#2
		     ;#1
	        }
	        ;##
	}
	off {
   		MOM_set_address_format X Coordinate
   		MOM_set_address_format Y Coordinate
   		MOM_set_address_format Z Coordinate
   		MOM_set_address_format I Coordinate
   		MOM_set_address_format J Coordinate
   		MOM_set_address_format K Coordinate
   	}
	default { ; }
    }

 return 0
}


#=============================================================
proc PB_start_of_program { } {
#=============================================================
 if { [llength [info commands PB_CMD_kin_start_of_program]] } { PB_CMD_kin_start_of_program ; }

 global type_postprocessors
 if {[info exists type_postprocessors]} {
     if {"HEIDENHAIN"==[string toupper $type_postprocessors]} { set type_postprocessors  "HEIDENHAIN" }
 } else {  set type_postprocessors "ISO" }

 global mom_warning_info
 set mom_warning_info "\n  TNC 360 ( $type_postprocessors).\n"
 append mom_warning_info "\t  TNC - 360:\n\t  :   ,  , , ;\n"
 append mom_warning_info "\t   :   -,  (  ),\n"
 append mom_warning_info "\t                           ,       \n"
 MOM_output_to_listing_device $mom_warning_info  ; #  MOM_catch_warning ;

#------- These are to be obtained for the user before Postprocessing
   MOM_force Once G_mode

   MOM_force ONCE Text G ;
   MOM_set_seq_off
   MOM_do_template start_of_program
   MOM_set_seq_on
}

#==================================================================
proc MOM_end_of_program {} {
#==================================================================
# This procedure is executed at the end of the program after all # the paths are traversed.
  global mom_sys_rewind_stop_code  mom_rewind_end_of_program
  global mom_machine_time
  global mom_warning_info
  global mom_seqnum

  global mom_time
  set tmpTime [ expr $mom_machine_time ] ;
  set timeMes "; Time PRG =[format "%.2f" $tmpTime ] min "
  set tm1 [ expr int($tmpTime/60.0) ]
  set tm2 [ expr ($tmpTime/60.0 - $tm1) * 60.0 ]
  set str2 [format " ( %.0f hour %.0f min )" $tm1 $tm2]
  if {$tm1!=0} { append timeMes $str2 ; }
  if {[info exists mom_time]} {
    if {$mom_time=="ON"} { MOM_output_literal "$timeMes" }
  }
  set mom_warning_info "\n   =[format "%.2f" $tmpTime ] min,$str2 \n"
  MOM_output_to_listing_device $mom_warning_info  ; #  MOM_catch_warning ;

  MOM_do_template end_off_path
  MOM_do_template  program_end_code

        global divide_flag max_length accom_length
 	if {$divide_flag=="OFF"} {
           if { [info exists accom_length] } {
           	if {$accom_length>=$max_length} {
           	  set tmp1 [ format "%.0f" $accom_length ]
      		  set mom_warning_info "\nWARNING:    = $tmp1 !!! \n\t   Heidenhain 'TNC-360',  FCQV NC63\n"
      		  append mom_warning_info "\t   -       4000  (    ),~64 \n"
                  append mom_warning_info "\t     4000 -           .\n"
                  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
                  ; # MOM_abort "\n$mom_warning_info\n"
                  ; #    ,    x  4000
                  global  mom_event_handler_file_name
                  set strDir [file dirname $mom_event_handler_file_name]
                  set strMes [file join $strDir "mesnumtcl.tcl" ]
                  if {[file exists $strMes ]} {
                   ; # source $strMes
                   set my_wish "[MOM_ask_env_var UGII_CAM_AUXILIARY_DIR]ugwish.exe"
                   exec $my_wish $strMes -- $mom_warning_info
                   }
                  ;#
           	}
           }
	}

  MOM_force ONCE Text G ;
  set mom_seqnum "9999"
  MOM_do_template start_of_program
  MOM_set_seq_off

  LIST_FILE_TRAILER ; #**** The following procedure lists the tool list with time in commentary data
  CLOSE_files ; #**** The following procedure closes the warning and listing files

}


#==================================================================
proc MOM_start_of_path {} {
#==================================================================
# This procedure is executed at the begining of each path in the task
# It gets called after all the general parameters of the path are loaded
# like the path name and the tool name.
#
  global mom_tool_name   mom_path_name   mom_operation_name
  global mom_warning_info

  global pb_start_of_program_flag
  if {$pb_start_of_program_flag == 0} {
     PB_start_of_program ;
     set pb_start_of_program_flag 1
   }

  global first_linear_move ; set first_linear_move 0
  TOOL_SET MOM_start_of_path

  if { [llength [info commands PB_CMD_kin_start_of_path]] } { PB_CMD_kin_start_of_path  }

  ;# MOM_force once X Y Z F

  global mom_imported_path_status
  global mom_kin_arc_output_mode
  if {[info exists mom_imported_path_status]==1} {
     if {"YES"==[string toupper "$mom_imported_path_status"]} {
       set mom_warning_info "WARNING: =$mom_operation_name -   ( -   !)"
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
       set mom_kin_arc_output_mode  "QUADRANT"  ;   #QUADRANT  FULL_CIRCLE , LINEAR
      } else {
        set mom_kin_arc_output_mode  "FULL_CIRCLE"  ;   #QUADRANT  FULL_CIRCLE , LINEAR
       }
     MOM_reload_kinematics
   }

  global var_stop ; set var_stop 0;
  global circleNumber ; set circleNumber 0; #       (   )
  global var_start_of_path_num ;incr var_start_of_path_num ; #   

}


  set accumulated_time 0
#==================================================================
proc MOM_end_of_path {} {
#==================================================================
# This procedure is executed at the end of a path.
  global mom_operation_name    mom_path_name
  global mom_machine_time
  global accumulated_time

  set op_time [expr $mom_machine_time - $accumulated_time]
  global mom_time
  if {[info exists mom_time]} {
    if {$mom_time=="ON"} {
      global  var_start_of_path_num  ; #   
      set tmpTime  [ expr $op_time ] ; # /2.54
      MOM_output_literal ";Time ($var_start_of_path_num)=[format "%.2f" $tmpTime ] min)"
     }
  }
  set accumulated_time $mom_machine_time

}


#==================================================================
proc MOM_initial_move {} {
#==================================================================
# This procedure is executed for the initial move of each operation.  It
# assumes the tool is moving from a safe position at rapid to the start of
# the operation.
  global mom_motion_type  mom_motion_event

  global initial_move ;   set initial_move 1
  global spindle_first is_from
  set  is_from  ""

#------- These are New Tool Start blocks:- Move to Tool Change Positon

 MOM_force once G_motion G_feed G_mode ; #Z G_cutcom
# MOM_force once M_spindle ; # X Y Z M_cool

  global mom_coolant_mode  mom_coolant_status
  if {[info exists mom_coolant_mode]} {  set mom_coolant_status $mom_coolant_mode  }

   global mom_programmed_feed_rate  mom_feed_rate  mom_kin_rapid_feed_rate
   if { [EQ_is_equal $mom_programmed_feed_rate 0.0] || [EQ_is_ge $mom_feed_rate $mom_kin_rapid_feed_rate ] } {
      MOM_rapid_move
   } else {
        if {$mom_motion_event == "rapid_move"} { MOM_rapid_move; return ;  }
  	if {$mom_motion_type == "RAPID"} { MOM_rapid_move ; return ; }
      MOM_linear_move
   }
}


#==================================================================
proc MOM_first_move {} {
#==================================================================
  global mom_motion_event
  catch { COOLANT_SET ; CUTCOM_SET ; SPINDLE_SET ; RAPID_SET ;  MODES_SET ; } strErr ; MOM_log_message "event MOM_first_move - $strErr"
  MOM_force once  G_motion G_mode
  catch {MOM_$mom_motion_event} strErr ;  MOM_log_message "event MOM_first_move - $strErr"
}


#########################################################################################
#==================================================================
proc MOM_before_motion {} {
#==================================================================
  global mom_motion_type  mom_current_motion  mom_motion_event

  global g_plane ;  set g_plane [CHECK_PLANE]

  FEEDRATE_SET

}


#==================================================================
proc MOM_linear_move {} {
#==================================================================
# This procedure is executed for each linear move. It gets called after
# the positioning varables, position and tool axis (X,Z,TAX,TAY,TAZ) are loaded.
  global mom_cutcom_mode  mom_cutcom_status
  global mom_motion_event
  global mom_motion_distance  mom_tool_diameter
  global mom_small_steps  mom_smooth_path
  global cutcom_on

 global mom_programmed_feed_rate
 if {$mom_motion_event == "rapid_move"} { MOM_rapid_move ;   return ; }

# if {[info exists mom_cutcom_mode]} {  set mom_cutcom_status $mom_cutcom_mode ;  }

  outDec on
        if {$mom_small_steps == "ON" 	&&
		$mom_cutcom_status == "ON" &&
			$mom_motion_distance <= $mom_tool_diameter} {
			    MOM_do_template feedrate_tnc
		   	    MOM_do_template linear_move_m97
	} elseif {$mom_smooth_path == "ON"} {
		if {$cutcom_on == 1} {
			MOM_do_template linear_move
			set cutcom_on 0
		} else {
			MOM_do_template feedrate_tnc
			MOM_do_template linear_move_m90
		 }
	} else  {
		MOM_do_template linear_move
	}
  outDec off

}


#==================================================================
proc MOM_circular_move {} {
#==================================================================
# This procedure is executed for each circular move. It gets called after
# the circle data are loaded (end point and center).
  global mom_pos_arc_axis    mom_pos  mom_pos_arc_plane  mom_pos_arc_center
  global mom_output_mode
  global mom_cutcom_mode   mom_cutcom_status
  CIRCLE_SET
  global g_plane ;  set g_plane [CHECK_PLANE]

    set xc  $mom_pos_arc_center(0)
    set yc  $mom_pos_arc_center(1)
    set zc  $mom_pos_arc_center(2)

  global xc_Prev yc_Prev zc_Prev
  global circleNumber ; incr circleNumber ;

 if {$circleNumber!=1} {
   switch $g_plane {
     "17" { set fl 0
     	  MOM_suppress once K Z; MOM_force once  I J X Y; # MOM_force always  I J ;
     	  if {[EQ_is_equal $xc $xc_Prev]} { incr fl ; }
     	  if {[EQ_is_equal $yc $yc_Prev]} { incr fl ; }
     	  if {[EQ_is_equal $fl 2]} { MOM_suppress once  I J ; }
    	}
     "18" { set fl 0
          MOM_suppress once J Y; MOM_force once  I K X Z; # MOM_force always  K I ;
     	  if {[EQ_is_equal $xc $xc_Prev]} { incr fl ; }
     	  if {[EQ_is_equal $zc $zc_Prev]} { incr fl ; }
     	  if {[EQ_is_equal $fl 2]} { MOM_suppress once  K I ; }
     	}
     "19" { set fl 0
     	  MOM_suppress once I X; MOM_force once  J K Y Z; # MOM_force always  J K ;
     	  if {[EQ_is_equal $yc $yc_Prev]} { incr fl ; }
     	  if {[EQ_is_equal $zc $zc_Prev]} { incr fl ; }
     	  if {[EQ_is_equal $fl 2]} { MOM_suppress once  K J ; }
     	}
   }
  } else {
      switch $g_plane {
        "17" { MOM_suppress once K Z ; MOM_force once  I J X Y; }
        "18" { MOM_suppress once J Y; MOM_force once  I K X Z; }
        "19" { MOM_suppress once I X; MOM_force once  J K Y Z; }
       }
    }

    outDec on
    MOM_force once G_motion
    MOM_do_template circle
    outDec off

    set xc_Prev  $mom_pos_arc_center(0)
    set yc_Prev  $mom_pos_arc_center(1)
    set zc_Prev  $mom_pos_arc_center(2)

  return
}


#=============================================================
proc PB_FORCE { option args } {
#=============================================================
   if { [info exists option] && [llength $args] } {
      lappend cmd MOM_force
      lappend cmd $option
      lappend cmd [join $args]
      eval { join $cmd }
   }
}

#=============================================================
proc PB_SET_RAPID_MOD { mod_list blk_list ADDR NEW_MOD_LIST } {
#=============================================================
  upvar $ADDR addr
  upvar $NEW_MOD_LIST new_mod_list
  global mom_cycle_spindle_axis traverse_axis1 traverse_axis2


   set new_mod_list [list]

   foreach mod $mod_list {
      switch $mod {
         "rapid1" {
            set elem $addr($traverse_axis1)
            if { [lsearch $blk_list $elem] >= 0 } {
               lappend new_mod_list $elem
            }
         }
         "rapid2" {
            set elem $addr($traverse_axis2)
            if { [lsearch $blk_list $elem] >= 0 } {
               lappend new_mod_list $elem
            }
         }
         "rapid3" {
            set elem $addr($mom_cycle_spindle_axis)
            if { [lsearch $blk_list $elem] >= 0 } {
               lappend new_mod_list $elem
            }
         }
         default {
            set elem $mod
            if { [lsearch $blk_list $elem] >= 0 } {
               lappend new_mod_list $elem
            }
         }
      }
   }
}

#==================================================================
proc MOM_rapid_move {} {
#==================================================================
# This procedure is executed for each rapid move. It gets called after
# the positioning varables, position and tool axis (X,Z,TAX,TAY,TAZ) are loaded.
  global mom_coolant_mode   mom_coolant_status
  global mom_motion_type

  catch { if {[info exists mom_coolant_mode]} { set mom_coolant_status $mom_coolant_mode ; } }

  global rapid_spindle_inhibit rapid_traverse_inhibit
  global spindle_first is_from
  global mom_cycle_spindle_axis traverse_axis1 traverse_axis2
  global mom_motion_event
  set is_from ""
  set aa(0) X ; set aa(1) Y ; set aa(2) Z
  RAPID_SET

  outDec on

   set rapid_spindle_blk { G_motion G_mode X Y Z}
   set rapid_spindle_x_blk { G_motion G_mode X}
   set rapid_spindle_y_blk { G_motion G_mode Y}
   set rapid_spindle_z_blk { G_motion G_mode Z}
   set rapid_traverse_blk {G_motion G_mode X Y Z M_spindle}
   set rapid_traverse_xy_blk {G_motion G_mode X Y M_spindle}
   set rapid_traverse_yz_blk {G_motion G_mode Y Z M_spindle}
   set rapid_traverse_xz_blk {G_motion G_mode X Z M_spindle}
   set rapid_traverse_mod {}
   set rapid_spindle_mod {}

   global mom_sys_control_out mom_sys_control_in
   set co "$mom_sys_control_out"
   set ci "$mom_sys_control_in"

   switch $mom_cycle_spindle_axis {
      0 {
         if [llength $rapid_spindle_x_blk] {
            set spindle_block  rapid_spindle_x
            PB_SET_RAPID_MOD $rapid_spindle_mod $rapid_spindle_x_blk aa mod_spindle
         } else {
            set spindle_block  ""
         }
         if [llength $rapid_traverse_yz_blk] {
            set traverse_block rapid_traverse_yz
            PB_SET_RAPID_MOD $rapid_traverse_mod $rapid_traverse_yz_blk aa mod_traverse
         } else {
            set traverse_block  ""
         }
      }

      1 {
         if [llength $rapid_spindle_y_blk] {
            set spindle_block  rapid_spindle_y
            PB_SET_RAPID_MOD $rapid_spindle_mod $rapid_spindle_y_blk aa mod_spindle
         } else {
            set spindle_block  ""
         }
         if [llength $rapid_traverse_xz_blk] {
            set traverse_block rapid_traverse_xz
            PB_SET_RAPID_MOD $rapid_traverse_mod $rapid_traverse_xz_blk aa mod_traverse
         } else {
            set traverse_block  ""
         }
      }

      2 {
         if [llength $rapid_spindle_z_blk] {
            set spindle_block  rapid_spindle_z
            PB_SET_RAPID_MOD $rapid_spindle_mod $rapid_spindle_z_blk aa mod_spindle
         } else {
            set spindle_block  ""
         }
         if [llength $rapid_traverse_xy_blk] {
            set traverse_block rapid_traverse_xy
            PB_SET_RAPID_MOD $rapid_traverse_mod $rapid_traverse_xy_blk aa mod_traverse
         } else {
            set traverse_block  ""
         }
      }

      default {
         set spindle_block  rapid_spindle
         set traverse_block rapid_traverse
         PB_SET_RAPID_MOD $rapid_spindle_mod $rapid_spindle_blk aa mod_spindle
         PB_SET_RAPID_MOD $rapid_traverse_mod $rapid_traverse_blk aa mod_traverse
      }
   }

   if {$spindle_first == "TRUE"} {
      if {$rapid_spindle_inhibit == "FALSE"} {
         if { $spindle_block != "" } {
            PB_FORCE Once $mod_spindle
            MOM_do_template $spindle_block
         } else {
            MOM_output_literal "$co Rapid Spindle Block is empty! $ci"
         }
      }
      if {$rapid_traverse_inhibit == "FALSE"} {
         if { $traverse_block != "" } {
            PB_FORCE Once $mod_traverse
            MOM_do_template $traverse_block
         } else {
            MOM_output_literal "$co Rapid Traverse Block is empty! $ci"
         }
      }
   } elseif {$spindle_first == "FALSE"} {
      if {$rapid_traverse_inhibit == "FALSE"} {
         if { $traverse_block != "" } {
            PB_FORCE Once $mod_traverse
            MOM_do_template $traverse_block
         } else {
            MOM_output_literal "$co Rapid Traverse Block is empty! $ci"
         }
      }
      if {$rapid_spindle_inhibit == "FALSE"} {
         if { $spindle_block != "" } {
            PB_FORCE Once $mod_spindle
            MOM_do_template $spindle_block
         } else {
            MOM_output_literal "$co Rapid Spindle Block is empty! $ci"
         }
      }
   } else {
      PB_FORCE Once $mod_traverse
      MOM_do_template rapid_traverse
   }

 outDec off

}

#==================================================================
proc MOM_cutcom_on {} {
#==================================================================
# This procedure is executed when the Cutcom command is activated.
  global mom_cutcom_status   mom_cutcom_mode
  CUTCOM_SET
  if {[info exists mom_cutcom_mode]} {   set mom_cutcom_status $mom_cutcom_mode ;  }
  global cutcom_on ;   set cutcom_on 1 ;
}

#==================================================================
proc MOM_cutcom_off {} {
#==================================================================
# This procedure is executed when the Cutcom command is deactivated.
  global mom_cutcom_status   mom_cutcom_mode
  CUTCOM_SET
  MOM_do_template cutcom_off
  global cutcom_on ;   set cutcom_on 0 ;
  catch {unset mom_cutcom_mode}
}


#==================================================================
proc MOM_spindle_rpm {} {
#==================================================================
# This procedure is executed when the Spindle command is activated.
  global mom_spindle_speed
  SPINDLE_SET
  MOM_force Once S M_spindle

  MOM_do_template spindle_rpm

}


#==================================================================
proc MOM_spindle_css {} {
#==================================================================
  MOM_spindle_rpm
}

#==================================================================
proc MOM_spindle_off {} {
#==================================================================
# This procedure is executed when the Spindle command is deactivated.
  MOM_force once M_spindle
  MOM_do_template spindle_off
}


#==================================================================
proc MOM_coolant_on {} {
#==================================================================
# This procedure is executed when the Coolant command is activated.
 global mom_coolant_status  mom_coolant_mode
 COOLANT_SET
 set mom_coolant_status $mom_coolant_mode
}


#==================================================================
proc MOM_coolant_off {} {
#==================================================================
# This procedure is executed when the Coolant command is deactivated.
  global mom_coolant_status  mom_coolant_mode
  COOLANT_SET
  MOM_force once M_cool
  MOM_do_template coolant_off
  catch { unset mom_coolant_mode }
}


#==================================================================
proc MOM_tool_change {} {
#==================================================================
# This procedure is executed at any time there is a tool change.
# It gets called after the tool change data is loaded like the tool number
# and the register numbers.
}

#==================================================================
proc MOM_first_tool {} {
#==================================================================
# This procedure is executed for the first operation, with a tool, in the task.
 global mom_tool_change_type mom_manual_tool_change
 global mom_update_post_cmds_from_tool

 global  first_tool ;set first_tool 1 ; #  1- 
 MOM_load_tool

}

#==================================================================
proc MOM_load_tool {} {
#==================================================================
# This procedure is executed for the LOAD/TOOL command.  All output for the LOAD
# command will take place with the initial move.
   PB_CMD_load_tools
   MOM_force Once G F ;
}


#==================================================================
proc PB_CMD_load_tools { } {
#==================================================================
  global mom_tool_change_type mom_manual_tool_change mom_tool_text
  global mom_tool_name    mom_tool_diameter
  global mom_tool_corner1_radius mom_tool_corner2_radius
  global mom_motion_event  mom_cycle_spindle_axis
  global mom_from_pos mom_pos  mom_prev_pos
  global mom_update_post_cmds_from_tool
  global mom_warning_info
  global mom_group_name mom_parent_group_name mom_template_subtype mom_template_type

  global g_plane ;  set g_plane [CHECK_PLANE]

  global mom_imported_path_status
  if {[info exists mom_imported_path_status]==1} {
      if {"NO"==[string toupper "$mom_imported_path_status"]} {
            set tmpTool [expr $mom_tool_diameter/2.0]
            set mom_tool_text  [format "; D=%.2f" $mom_tool_diameter]
            if {[info exists mom_tool_corner1_radius]} {
                append mom_tool_text  [format " r1=%.2f" $mom_tool_corner1_radius ]
             }
            if {[info exists mom_tool_corner2_radius]} {
                append mom_tool_text  [format " r2=%.2f" $mom_tool_corner2_radius]
             }
            set mom_tool_text  ""
       }
   }

  if {[info exists mom_tool_change_type]} {
    switch $mom_tool_change_type {
         MANUAL {
                  MOM_do_template tool_change_manual
                  MOM_suppress ONCE Text
                  if {[info exists mom_tool_text]} {
                    if {$mom_tool_text != ""} { MOM_force Once Text }
                  }
                  MOM_force Once T G_plane S
                  MOM_force Once M_spindle
                  MOM_do_template  tool_call
                 }
         AUTO {
		;#     
                 global  first_tool  var_start_of_path_num ;
                 if {$var_start_of_path_num!=1} {
                     global var_stop ;
                     if {[info exists var_stop]==1} {
                       if {1==$var_stop} { set var_stop 1 ; } else { set var_stop 0 ; }
                     } else { set var_stop 0 ; }
                     if {0==$var_stop} {
                       ;# set mom_warning_info "WARNING:      ( M6  ) mom_group_name=$mom_group_name mom_parent_group_name=$mom_parent_group_name mom_template_subtype=$mom_template_subtype mom_template_type=$mom_template_type"
                       ;# MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
                       ;# MOM_do_template tool_change_manual ;
                     }
                     set var_stop 0 ;
                 }
		;#
		 MOM_suppress ONCE Text
                 if {[info exists mom_tool_text]} {
                   if {$mom_tool_text != ""} { MOM_force Once Text }
                 }
                 MOM_force Once T G_plane S ;
                 MOM_do_template  tool_call
                }
   }
  } elseif {[info exists mom_manual_tool_change]} {
    if {$mom_manual_tool_change == "TRUE"} {
         MOM_do_template tool_change_manual
         MOM_suppress ONCE Text
         if {[info exists mom_tool_text]} {
              if {$mom_tool_text != ""} { MOM_force Once Text }
         }
         MOM_force Once T G_plane S ;
         MOM_do_template  tool_call
     }
  }

}

#==================================================================
proc MOM_length_compensation {} {
#==================================================================
# This procedure is executed when the Tool Length Compensation command is activated.
  global mom_warning_info
  set mom_warning_info "WARNING:     "
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

#==================================================================
proc MOM_tool_preselect {} {
#==================================================================
# This procedure is executed when the Tool Preselect command is activated.
  global mom_tool_preselect_number mom_tool_number mom_next_tool_number  mom_tool_preselect_text

  global mom_warning_info
  set mom_warning_info "WARNING:     'TNC-360'   \[ tool_preselect\]"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;

  MOM_suppress ONCE Text
  if {[info exists mom_tool_preselect_text]} {
       if {$mom_tool_preselect_text != ""} { MOM_force Once Text }
  }
  MOM_force once  G T
  MOM_do_template  tool_preselect
}


#==================================================================
proc  MOM_zero {} {
#==================================================================
# This procedure is executed when the ZERO command is activated.
# G53 P_ ; G54 X_ Y_ Z_ ; G54 X0 Y0 Z0
   global mom_warning_info
   global mom_imported_path_status
   if {[info exists mom_imported_path_status]==1} {
     if {"YES"==[string toupper "$mom_imported_path_status"]} {
       set mom_warning_info "WARNING:       G53 P_ , G54 X_ Y_ Z_ "
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
       return ;
      }
   }

   set mom_warning_info "WARNING:       G53 P_ , G54 X_ Y_ Z_ "
   MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
   return ;

 global mom_sys_wcs_change_code mom_work_coordinate_number
 if {[info exists mom_work_coordinate_number]} {
   if {$mom_work_coordinate_number==0} {
	;
     } else {
      MOM_output_literal "G53 P0"
     }
    } else {
    set mom_work_coordinate_number  0 ;
  }
#
   MOM_force once G cycle_datum_shift
   MOM_do_template datum_shift_table

}


#==================================================================
proc MOM_opstop {} {
#==================================================================
# This procedure is executed when the Optional Stop command is activated.
  MOM_do_template opstop
}


#==================================================================
proc MOM_stop {} {
#==================================================================
# This procedure is executed when the Program Stop command is activated.
    global var_stop ; set var_stop 1;
    MOM_do_template stop
}


#=============================================================
proc  MOM_pprint {} {
#___ This procedure is executed when the Text command is activated.
#=============================================================
 global mom_pprint
 global mom_warning_info

 if {[info exists mom_pprint]} {
  if {$mom_pprint!=""} {
     set var1 "" ;
     set st [regexp -nocase -- {[-]} $mom_pprint var1]
     if {$st!=0} {
       set mom_warning_info "WARNING:       "
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
     }
     MOM_output_literal ";$mom_pprint"
   }
  }

}

#==================================================================
proc MOM_operator_message {} {
#==================================================================
# This procedure is executed when the Operator Message command is activated.
 global  mom_operator_message  mom_operator_message_defined
 global mom_warning_info

   if {[info exists mom_operator_message]} {
    if {$mom_operator_message!=""} {
     set var1 "" ;
     set st [regexp -nocase -- {[-]} $mom_operator_message  var1]
     if {$st!=0} {
       set mom_warning_info "WARNING:     -  ..."
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
     }
     MOM_output_literal ";$mom_operator_message"
     }
    }

}


#==================================================================
proc MOM_delay {} {
#==================================================================
# This procedure is executed when the Delay command is activated.
  global mom_sys_delay_param mom_delay_value
  global mom_delay_revs mom_delay_mode delay_time
  global mom_warning_info

   # post builder provided format for the current mode:
    if {[info exists mom_sys_delay_param(${mom_delay_mode},format)] != 0} {
      MOM_set_address_format dwell $mom_sys_delay_param(${mom_delay_mode},format)
    }

    switch $mom_delay_mode {
      SECONDS {set delay_time $mom_delay_value}
      default {
         set delay_time $mom_delay_revs ;
         set mom_warning_info "WARNING:      . !"
         MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
      }
    }
   MOM_force ONCE G dwell
   MOM_do_template delay

}


#==================================================================
proc MOM_auxfun {} {
#==================================================================
# This procedure is executed when the Auxiliary command is activated.
 global mom_auxfun_text  mom_auxfun_text_defined
 MOM_suppress once Text
 if {[info exists  mom_auxfun_text]} {
    if {$mom_auxfun_text!=""} { MOM_force  once  Text }
   }
  MOM_do_template auxiliary
}


#==================================================================
proc MOM_prefun {} {
#==================================================================
# This procedure is executed when the Preparatory function command is activated.
 global mom_prefun_text  mom_prefun_text_defined
 MOM_suppress once Text
 if {[info exists mom_prefun_text]} {
    if {$mom_prefun_text!=""} { MOM_force  once  Text }
   }
  MOM_do_template preparatory
}


#==================================================================
proc MOM_set_modes {} {
#==================================================================
# This procedure is executed for the Set Mode command.
  global mom_output_mode
  global mom_arc_mode
  global mom_feed_rate_output_mode
  global mom_parallel_to_axis
  global mom_machine_mode
  global mom_modes_text

; # "ABSOLUTE", "INCREMENTAL"
  switch $mom_output_mode {
    ABSOLUTE { set isincr OFF }
    default  { set isincr ON }
  }
  MOM_incremental $isincr X Y Z
  MOM_force once  G_mode

# "LINEAR", "CIRCULAR"
  if {$mom_arc_mode == "LINEAR"} {
   } elseif {$mom_arc_mode == "CIRCULAR"} { ; }

# "OFF", "IPM", "IPR", "MMPM", "MMPR", "INVERSE"
  switch $mom_feed_rate_output_mode {
    MMPM { ; }
    OFF  { ; }
    default  {
       set mom_warning_info "WARNING:    (  MMPM)"
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
       set mom_feed_rate_output_mode  "MMPM"
     }
  }

# "ZAXIS", "WAXIS", "UAXIS"
  switch $mom_parallel_to_axis {
    ZAXIS { ; }
    default  {
       set mom_warning_info "WARNING:     Z (   Z)"
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
       set mom_parallel_to_axis  "ZAXIS"
     }
  }

# "MILL", "TURN", "PUNCH", "LASER", "TORCH", "WIRE"
  switch $mom_machine_mode {
    MILL { ; }
    default  {
       set mom_warning_info "WARNING:       MILL (\[$mom_machine_mode\]) (  MILL)"
       MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
       set  mom_machine_mode  "MILL"
     }
  }

}


#==================================================================
proc MOM_text {} {
#==================================================================
# This procedure is executed when the Text command is activated.
  global mom_user_defined_text mom_record_fields mom_field_count mom_record_text
  global mom_Instruction
  global mom_sys_control_out mom_sys_control_in
  global mom_pprint mom_pprint_defined
  global mom_operator_message mom_operator_message_defined
  global mom_warning_info


  switch [string toupper $mom_record_fields(0)] {
  "SET" { ; }
  "PPRINT" {
           set mom_pprint_defined 1
           set mom_pprint $mom_record_text
           MOM_pprint
           }
  "DISPLY" {
           set mom_operator_message_defined 1
           set mom_operator_message $mom_record_text
           MOM_operator_message
          }
  "INSERT" {
           if {[info exists mom_record_fields(1)]}  {
              set mom_Instruction [string toupper $mom_record_text ] ; #$mom_record_fields(1)] ;
              MOM_insert
              set mom_warning_info "WARNING:    'INSERT/$mom_Instruction' -  "
              MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
           }
         }
  "PARTNO" { ; #   ()
           global partno_program
           set partno_program  "" ;
           set str "User Defined"
           if {[info exists  mom_record_fields(1)]} {
             set partno_program $mom_record_text ; # $mom_record_fields(1)
           }  else  {
              set RNseed [clock clicks] ;
              set RNseed [expr (30903 * ($RNseed & 65535) + ($RNseed>>16))]
              set RNseed [expr (($RNseed & 65535)/65535.0)]
              set RNseed [expr $RNseed*100000.0]
              set RNseed [expr int($RNseed)]
              set partno_program [format "%5d" $RNseed]
              set str "."
           }
           set mom_warning_info "<!--\n PARTNO-> \[  () - !   \]=$partno_program ($str)\n-->"
           MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
         }
  "REWIND" {
            global mom_rewind_end_of_program ; set mom_rewind_end_of_program 1 ;
            set mom_warning_info "<!--\n REWIND-> \[ \]=M30;\n-->"
            MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
           }
  "MACALL" { ; #  
          }
  "MACND" { ; }
  "MACST" { ; }
  "SLOWDN" { ; }
  "TIME" {
          global mom_time ;
          set  mom_time  ON ;
          if {[info exists mom_record_fields(1)]} {
           if {$mom_record_fields(1) == "OFF"} { set  mom_time  OFF ; }
           }
          set mom_warning_info "<!--\n TIME-> TIME \[ \]\n =$mom_time ;\n-->"
          MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
         }
  "ZERO" {
          global  mom_work_coordinate_number
          set mom_work_coordinate_number $mom_record_fields(1)
           MOM_zero
         }
   default
         {
           MOM_output_literal $mom_user_defined_text
         }
  }
}


#==================================================================
proc MOM_path_warning { } {
#==================================================================
# This procedure is executed when the in CLSF file has Inconsistent axis.
  global mom_seqnum
  global mom_warning_info
  global mom_path_warning
  if {[info exists mom_path_warning]} {
     if {[string length $mom_path_warning]!=0} {
     	set tmpNum [format "%.0f" $mom_seqnum]
        set mom_warning_info "!! :     ( ?     ) = $mom_path_warning ( =$tmpNum)"
     	MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
     	}
   }
  #MOM_abort $mom_warning_info
}


#==================================================================
proc MOM_sequence_number {} {
#==================================================================
# This procedure is executed for the Sequence Number command.
  SEQNO_SET
}


###################################################################################################

#=============================================================
proc MOM_rapid_move_cycle { } {
#=============================================================
   	global mom_pos mom_prev_pos mom_cycle_rapid_to mom_cycle_rapid_to_pos

   	set mom_pos(2) $mom_cycle_rapid_to_pos(2)
  	if {$mom_pos(2) <= $mom_prev_pos(2) } {
   		MOM_do_template rapid_cycle_xy
   		MOM_do_template rapid_cycle_z
   	} else {
   		MOM_do_template rapid_cycle_z
   		MOM_do_template rapid_cycle_xy
   	}
}

proc MOM_cycle_plane_change {} {
#==================================================================
# This procedure is executed at the end of cycle motion when there is a cycle
# move occuring at a higher level than the previous position.
   MOM_rapid_move_cycle
}

proc MOM_cycle_off {} {
#==================================================================
# This procedure is executed at the end of cycle motion.
}

proc MOM_drill {} {
#==================================================================
# This procedure is executed at the begining of a drill cycle.
  ; # DEF SET UP DEPTH PECKG DWELL F
  MOM_do_template cycle_drill
}

proc MOM_drill_move {} {
#==================================================================
# This procedure is executed for the goto of a drill cycle.
  MOM_rapid_move_cycle
  MOM_do_template cycle_call
}

proc MOM_drill_dwell {} {
#==================================================================
# This procedure is executed at the begining of a drill dwell cycle.
   ; # DEF SET UP DEPTH PECKG DWELL F
   MOM_do_template cycle_drill_dwell
}

proc MOM_drill_dwell_move {} {
#==================================================================
# This procedure is executed for the goto of a drill dwell cycle.
  MOM_rapid_move_cycle
  MOM_do_template cycle_call
}

proc MOM_drill_counter_sink {} {
#==================================================================
# This procedure is executed at the begining of a drill cycle.
  ; # DEF SET UP DEPTH PECKG DWELL F
  MOM_drill
}

proc MOM_drill_counter_sink_move {} {
#==================================================================
# This procedure is executed for the goto of a drill cycle.
  MOM_drill_move
}

proc MOM_drill_csink_dwell {} {
#==================================================================
# This procedure is executed at the begining of a drill dwell cycle.
  ; # DEF SET UP DEPTH PECKG DWELL F
  MOM_drill_dwell
}

proc MOM_drill_csink_dwell_move {} {
#==================================================================
# This procedure is executed for the goto of a drill dwell cycle.
  MOM_drill_dwell_move
}

proc MOM_drill_deep {} {
#==================================================================
# This procedure is executed at the begining of a deep cycle.
  ; # DEF SET UP DEPTH PECKG DWELL F
  MOM_do_template cycle_drill_deep
}

proc MOM_drill_deep_move {} {
#==================================================================
# This procedure is executed for the goto of a  deep cycle.
  MOM_rapid_move_cycle
  MOM_do_template cycle_call
}

proc MOM_drill_break_chip {} {
#==================================================================
# This procedure is executed at the begining of a brkchp cycle.
   MOM_do_template cycle_breakchip
}

proc MOM_drill_break_chip_move {} {
#==================================================================
# This procedure is executed for the goto of a  brkchp cycle.
  MOM_rapid_move_cycle
  MOM_do_template cycle_call
}

proc MOM_tap {} {
#==================================================================
# This procedure is executed at the begining of a tap cycle.

}

proc MOM_tap_move {} {
#==================================================================
# This procedure is executed for the goto of a tap cycle.
  MOM_rapid_move_cycle

  global mom_spindle_direction
  global cycle_tap_code
  global mom_sys_cycle_tap_code_left
  global mom_sys_cycle_tap_code

  if {$mom_spindle_direction == "CCLW"} {
      MOM_set_address_format G_motion Coordinate
      set cycle_tap_code $mom_sys_cycle_tap_code_left
  } else {
      set cycle_tap_code $mom_sys_cycle_tap_code
  }
  MOM_do_template cycle_tap
}

proc MOM_bore {} {
#==================================================================
# This procedure is executed at the begining of a bore cycle.
  global mom_warning_info
  set mom_warning_info "\nWARNING:       \[bore\]\n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

proc MOM_bore_move {} {
#==================================================================
# This procedure is executed for the goto of a bore cycle.
   MOM_linear_move
}

proc MOM_bore_dwell {} {
#==================================================================
# This procedure is executed at the begining of a bore cycle.
  global mom_warning_info
  set mom_warning_info "\nWARNING:         \[bore_dwell\]\n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

proc MOM_bore_dwell_move {} {
#==================================================================
# This procedure is executed for the goto of a bore dwell cycle.
    MOM_linear_move
}

proc MOM_bore_drag {} {
#==================================================================
# This procedure is executed at the begining of a bore drag cycle.
  global mom_warning_info
  set mom_warning_info "\nWARNING:   ()     \[bore_drag\]\n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

proc MOM_bore_drag_move {} {
#==================================================================
# This procedure is executed for the goto of a bore_drag cycle.
    MOM_linear_move
}

proc MOM_bore_no_drag {} {
#==================================================================
# This procedure is executed at the begining of a bore nodrag cycle.
  global mom_warning_info
  set mom_warning_info "\nWARNING:   C   \[bore_no_drag\]    \n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

proc MOM_bore_no_drag_move {} {
#==================================================================
# This procedure is executed for the goto of a bore nodrag cycle.
    MOM_linear_move
}

proc MOM_bore_back {} {
#==================================================================
# This procedure is executed at the begining of a bore back cycle.
  global mom_warning_info
  set mom_warning_info "\nWARNING:    \[bore_back\]    \n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

proc MOM_bore_back_move {} {
#==================================================================
# This procedure is executed for the goto of a back bore cycle.
	MOM_linear_move
}

proc MOM_bore_manual {} {
#==================================================================
# This procedure is executed at the begining of a bore manual cycle.
  global mom_warning_info
  set mom_warning_info "\nWARNING:   C   \[bore_manual\]    \n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

proc MOM_bore_manual_move {} {
#==================================================================
# This procedure is executed for the goto of a manual bore cycle.
	MOM_linear_move
}

proc MOM_bore_manual_dwell {} {
#==================================================================
# This procedure is executed at the begining of a bore manual dwell bore cycle.
  global mom_warning_info
  set mom_warning_info "\nWARNING:   C     \[bore_manual_dwell\]    \n     Point-to-Point\n"
  MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

proc MOM_bore_manual_dwell_move {} {
#==================================================================
# This procedure is executed for the goto of a manual dwell borecycle.
	MOM_linear_move
}


#################################################################################################

#=============================================================
proc MOM_rotation_cycle { } {
#=============================================================
   global mom_rotation mom_rotation_angle

   set mom_warning_info "WARNING:  (CYCL DEF 10.0 ROTATION)  -    () !"
   MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

#=============================================================
proc MOM_datum_shift_cycle { } {
#=============================================================
   global mom_datum_shift

   set mom_warning_info "WARNING:  (CYCL DEF 7.0 DATUM SHIFT)  -    () !"
   MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

#=============================================================
proc MOM_scaling_cycle { } {
#=============================================================
   global mom_scaling  mom_scaling_factor

   set mom_warning_info "WARNING:  (CYCL DEF 11.0 SCALING)  -    () !"
   MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}

#=============================================================
proc MOM_small_step_contour { } {
#=============================================================
	global mom_small_steps

}

#=============================================================
proc MOM_smooth_tool_path { } {
#=============================================================
	global mom_smooth_path
}
#=============================================================
proc MOM_tolerance_cycle { } {
#=============================================================
	global mom_tolerance_on_off  tolerances
	global mom_operation_type mom_inside_outside_tolerances
  	set temp [expr $mom_inside_outside_tolerances(0) + $mom_inside_outside_tolerances(1)]
  	set tolerances [expr $temp/2]
   set mom_warning_info "WARNING:  G62 (CYCL DEF 32.0 TOLERANCE)  TNC 360 -   !"
   MOM_output_to_listing_device $mom_warning_info  ;   MOM_catch_warning ;
}
#=============================================================
proc MOM_spindle_orient { } {
#=============================================================
	global mom_angle_orient
	MOM_force once Angle
	MOM_do_template spindle_orient
}

#=============================================================
proc  MOM_divide { } {
#=============================================================
# "DIVIDE"
	global mom_divide_by mom_length_of_program mom_live_time
	global mom_retract_to mom_offset_dist mom_retract_point_pos
	global max_length ; set max_length [expr double($mom_length_of_program) ]
        global divide_flag ; set divide_flag ON
}


###############################################################################
#------------------------------------------------------------------------------
# Local Procedures for use in this post
#------------------------------------------------------------------------------

proc CHECK_PLANE {} {

  global mom_tool_axis
  global mom_path_name

  global prev_error_path

#-------- Decision on plane of operation

  if {[EQ_is_equal $mom_tool_axis(2) 0.0]} {
      set g_plane 19
  } elseif {[EQ_is_equal $mom_tool_axis(2) 1.0]} {
      set g_plane 17
  } elseif {([EQ_is_gt $mom_tool_axis(2) 0.0] && \
                 [EQ_is_lt $mom_tool_axis(2) 1.0]) || \
                        [EQ_is_lt $mom_tool_axis(2) 0.0] } {
      set g_plane "NONE"

      set i [format "%-.7f" $mom_tool_axis(0)]
      set j [format "%-.7f" $mom_tool_axis(1)]
      set k [format "%-.7f" $mom_tool_axis(2)]

      # Check to output this error only once per path

      if {[info exists prev_error_path]} {
            if {$prev_error_path == $mom_path_name} { return $g_plane }
      }
      MOM_output_to_listing_device "!ERROR! PATH - $mom_path_name       ($i,$j,$k)"
      MOM_output_to_listing_device "3-axis Mill Turn machine        "
      MOM_output_to_listing_device "    ()!!"
      MOM_catch_warning ;

      set prev_error_path $mom_path_name
  } else {
      set g_plane 18
  }

  return $g_plane
}


proc SET_SIGN_OF_RADIUS {} {
  global mom_arc_radius   mom_arc_angle
  if {$mom_arc_angle > 180} {
      set mom_arc_radius [expr (-1.0 * $mom_arc_radius)]
  } else {
      set mom_arc_radius [expr abs($mom_arc_radius)]
  }
}
###############################################################################
